@@ -1,44 +1,7 @@
 /* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is the Firefox Sanitizer.
- *
- * The Initial Developer of the Original Code is
- * Ben Goodger.
- * Portions created by the Initial Developer are Copyright (C) 2005
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Ben Goodger <ben@mozilla.org>
- *   Giorgio Maone <g.maone@informaction.com>
- *   Johnathan Nightingale <johnath@mozilla.com>
- *   Drew Willcoxon <adw@mozilla.com>
- *   Ehsan Akhgari <ehsan.akhgari@gmail.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 const Cc = Components.classes;
 const Ci = Components.interfaces;
@@ -79,11 +42,18 @@ var gSanitizePromptDialog = {
 
     var s = new Sanitizer();
     s.prefDomain = "privacy.cpd.";
-    for (let i = 0; i < this.sanitizePreferences.childNodes.length; ++i) {
-      var preference = this.sanitizePreferences.childNodes[i];
-      var name = s.getNameFromPreference(preference.name);
-      if (!s.canClearItem(name)) 
-        preference.disabled = true;
+
+    let sanitizeItemList = document.querySelectorAll("#itemList > [preference]");
+    for (let i = 0; i < sanitizeItemList.length; i++) {
+      let prefItem = sanitizeItemList[i];
+      let name = s.getNameFromPreference(prefItem.getAttribute("preference"));
+      s.canClearItem(name, function canClearCallback(aItem, aCanClear, aPrefItem) {
+        if (!aCanClear) {
+          aPrefItem.preference = null;
+          aPrefItem.checked = false;
+          aPrefItem.disabled = true;
+        }
+      }, prefItem);
     }
 
     document.documentElement.getButton("accept").label =
@@ -139,12 +109,25 @@ var gSanitizePromptDialog = {
     s.range = Sanitizer.getClearRange(this.selectedTimespan);
     s.ignoreTimespan = !s.range;
 
+    // As the sanitize is async, we disable the buttons, update the label on
+    // the 'accept' button to indicate things are happening and return false -
+    // once the async operation completes (either with or without errors)
+    // we close the window.
+    let docElt = document.documentElement;
+    let acceptButton = docElt.getButton("accept");
+    acceptButton.disabled = true;
+    acceptButton.setAttribute("label",
+                              this.bundleBrowser.getString("sanitizeButtonClearing"));
+    docElt.getButton("cancel").disabled = true;
     try {
-      s.sanitize();
+      s.sanitize().then(null, Components.utils.reportError)
+                  .then(() => window.close())
+                  .then(null, Components.utils.reportError);
     } catch (er) {
       Components.utils.reportError("Exception during sanitize: " + er);
+      return true; // We *do* want to close immediately on error.
     }
-    return true;
+    return false;
   },
 
   /**
@@ -308,11 +291,18 @@ var gSanitizePromptDialog = {
 
     var s = new Sanitizer();
     s.prefDomain = "privacy.cpd.";
-    for (let i = 0; i < this.sanitizePreferences.childNodes.length; ++i) {
-      var preference = this.sanitizePreferences.childNodes[i];
-      var name = s.getNameFromPreference(preference.name);
-      if (!s.canClearItem(name)) 
-        preference.disabled = true;
+
+    let sanitizeItemList = document.querySelectorAll("#itemList > [preference]");
+    for (let i = 0; i < sanitizeItemList.length; i++) {
+      let prefItem = sanitizeItemList[i];
+      let name = s.getNameFromPreference(prefItem.getAttribute("preference"));
+      s.canClearItem(name, function canClearCallback(aCanClear) {
+        if (!aCanClear) {
+          prefItem.preference = null;
+          prefItem.checked = false;
+          prefItem.disabled = true;
+        }
+      });
     }
 
     document.documentElement.getButton("accept").label =
@@ -681,6 +671,7 @@ var gContiguousSelectionTreeHelper = {
               createInstance(Ci.nsISupportsArray);
     var trans = Cc["@mozilla.org/widget/transferable;1"].
                 createInstance(Ci.nsITransferable);
+    trans.init(null);
     trans.setTransferData('dummy-flavor', null, 0);
     arr.AppendElement(trans);
     var reg = Cc["@mozilla.org/gfx/region;1"].
@@ -830,9 +821,6 @@ var gContiguousSelectionTreeHelper = {
    */
   _makeTreeView: function CSTH__makeTreeView(aProtoTreeView)
   {
-    var atomServ = Cc["@mozilla.org/atom-service;1"].
-                   getService(Ci.nsIAtomService);
-
     var view = aProtoTreeView;
     var that = this;
 
@@ -863,28 +851,29 @@ var gContiguousSelectionTreeHelper = {
 
     view._getCellProperties = view.getCellProperties;
     view.getCellProperties =
-      function CSTH_View_getCellProperties(aRow, aCol, aProps)
+      function CSTH_View_getCellProperties(aRow, aCol)
       {
         var grippyRow = that.getGrippyRow();
         if (aRow === grippyRow)
-          aProps.AppendElement(atomServ.getAtom("grippyRow"));
-        else if (aRow < grippyRow)
-          this._getCellProperties(aRow, aCol, aProps);
-        else
-          this._getCellProperties(aRow - 1, aCol, aProps);
+          return "grippyRow";
+        if (aRow < grippyRow)
+          return this._getCellProperties(aRow, aCol);
+
+        return this._getCellProperties(aRow - 1, aCol);
       };
 
     view._getRowProperties = view.getRowProperties;
     view.getRowProperties =
-      function CSTH_View_getRowProperties(aRow, aProps)
+      function CSTH_View_getRowProperties(aRow)
       {
         var grippyRow = that.getGrippyRow();
         if (aRow === grippyRow)
-          aProps.AppendElement(atomServ.getAtom("grippyRow"));
-        else if (aRow < grippyRow)
-          this._getRowProperties(aRow, aProps);
-        else
-          this._getRowProperties(aRow - 1, aProps);
+          return "grippyRow";
+
+        if (aRow < grippyRow)
+          return this._getRowProperties(aRow);
+
+        return this._getRowProperties(aRow - 1);
       };
 
     view._getCellText = view.getCellText;
