@@ -4,7 +4,7 @@
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 MOZ_APP_BASENAME=IceCat
-MOZ_APP_VENDOR=Mozilla
+MOZ_APP_VENDOR=GNU
 MOZ_UPDATER=1
 MOZ_PHOENIX=1
 
@@ -28,10 +28,10 @@ MOZ_DISABLE_EXPORT_JS=1
 MOZ_SAFE_BROWSING=1
 MOZ_SERVICES_COMMON=1
 MOZ_SERVICES_CRYPTO=1
-MOZ_SERVICES_FXACCOUNTS=1
-MOZ_SERVICES_HEALTHREPORT=1
-MOZ_SERVICES_METRICS=1
-MOZ_SERVICES_SYNC=1
+MOZ_SERVICES_FXACCOUNTS=0
+MOZ_SERVICES_HEALTHREPORT=0
+MOZ_SERVICES_METRICS=0
+MOZ_SERVICES_SYNC=0
 MOZ_APP_VERSION=$ICECAT_VERSION
 MOZ_EXTENSIONS_DEFAULT=" gio"
 # MOZ_APP_DISPLAYNAME will be set by branding/configure.sh
@@ -48,9 +48,9 @@ MOZ_APP_ID={ec8030f7-c20a-464f-9b0e-13a3a9e97384}
 # This should usually be the same as the value MAR_CHANNEL_ID.
 # If more than one ID is needed, then you should use a comma separated list
 # of values.
-ACCEPTED_MAR_CHANNEL_IDS=icecat-mozilla-esr
+ACCEPTED_MAR_CHANNEL_IDS=icecat-gnu-esr
 # The MAR_CHANNEL_ID must not contain the following 3 characters: ",\t "
-MAR_CHANNEL_ID=icecat-mozilla-esr
+MAR_CHANNEL_ID=icecat-gnu-esr
 MOZ_PROFILE_MIGRATOR=1
 MOZ_EXTENSION_MANAGER=1
 MOZ_APP_STATIC_INI=1
@@ -61,10 +61,11 @@ if test "$OS_TARGET" = "WINNT" -o "$OS_TARGET" = "Darwin"; then
 fi
 MOZ_WEBGL_CONFORMANT=1
 # Enable navigator.mozPay
-MOZ_PAY=1
+MOZ_PAY=0
 # Enable activities. These are used for FxOS developers currently.
 MOZ_ACTIVITIES=1
 MOZ_JSDOWNLOADS=1
 MOZ_WEBM_ENCODER=1
 # Enable exact rooting on desktop.
 export JSGC_USE_EXACT_ROOTING=1
+MOZ_DATA_REPORTING=0
