@@ -7,10 +7,9 @@ var gotTimeupdate = false;
 
 function startTest() {
   if (completed)
-    return false;
+    return;
 
   v.currentTime=seekTime;
-  return false;
 }
 
 function timeupdate() {
@@ -20,24 +19,22 @@ function timeupdate() {
 
 function seekStarted() {
   if (completed)
-    return false;
+    return;
 
   ok(v.currentTime >= seekTime - 0.1, "Video currentTime should be around " + seekTime + ": " + v.currentTime);
   v.addEventListener("timeupdate", timeupdate, false);
   startPassed = true;
-  return false;
 }
 
 function seekEnded() {
   if (completed)
-    return false;
+    return;
 
   var t = v.currentTime;
   ok(Math.abs(t - seekTime) <= 0.1, "Video currentTime should be around " + seekTime + ": " + t);
   ok(gotTimeupdate, "Should have got timeupdate between seeking and seekended");
   completed = true;
   finish();
-  return false;
 }
 
 v.addEventListener("loadedmetadata", startTest, false);
