@@ -1,41 +1,8 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* vim: set ts=2 sw=2 et tw=80: */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Original Author: David W. Hyatt (hyatt@netscape.com)
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /*
 
@@ -47,10 +14,8 @@
 #include "nsFocusManager.h"
 #include "nsIControllers.h"
 #include "nsIDOMDocument.h"
-#include "nsIDOMXULDocument.h"
-#include "nsIDOMHTMLDocument.h"
 #include "nsIDOMElement.h"
-#include "nsIDOMWindowInternal.h"
+#include "nsIDOMWindow.h"
 #include "nsIDOMXULElement.h"
 #include "nsIDocument.h"
 #include "nsPresContext.h"
@@ -61,27 +26,30 @@
 #include "nsRDFCID.h"
 #include "nsXULCommandDispatcher.h"
 #include "prlog.h"
-#include "nsIDOMEventTarget.h"
-#include "nsGUIEvent.h"
 #include "nsContentUtils.h"
 #include "nsReadableUtils.h"
 #include "nsCRT.h"
-#include "nsDOMError.h"
-#include "nsEventDispatcher.h"
+#include "nsError.h"
+#include "nsDOMClassInfoID.h"
+#include "mozilla/BasicEvents.h"
+#include "mozilla/EventDispatcher.h"
+#include "mozilla/dom/Element.h"
+
+using namespace mozilla;
 
 #ifdef PR_LOGGING
-static PRLogModuleInfo* gLog;
+static PRLogModuleInfo* gCommandLog;
 #endif
 
 ////////////////////////////////////////////////////////////////////////
 
 nsXULCommandDispatcher::nsXULCommandDispatcher(nsIDocument* aDocument)
-    : mDocument(aDocument), mUpdaters(nsnull)
+    : mDocument(aDocument), mUpdaters(nullptr)
 {
 
 #ifdef PR_LOGGING
-  if (! gLog)
-    gLog = PR_NewLogModule("nsXULCommandDispatcher");
+  if (! gCommandLog)
+    gCommandLog = PR_NewLogModule("nsXULCommandDispatcher");
 #endif
 }
 
@@ -90,8 +58,6 @@ nsXULCommandDispatcher::~nsXULCommandDispatcher()
   Disconnect();
 }
 
-NS_IMPL_CYCLE_COLLECTION_CLASS(nsXULCommandDispatcher)
-
 // QueryInterface implementation for nsXULCommandDispatcher
 
 DOMCI_DATA(XULCommandDispatcher, nsXULCommandDispatcher)
@@ -106,12 +72,14 @@ NS_INTERFACE_MAP_END
 NS_IMPL_CYCLE_COLLECTING_ADDREF(nsXULCommandDispatcher)
 NS_IMPL_CYCLE_COLLECTING_RELEASE(nsXULCommandDispatcher)
 
+NS_IMPL_CYCLE_COLLECTION_CLASS(nsXULCommandDispatcher)
+
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(nsXULCommandDispatcher)
   tmp->Disconnect();
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(nsXULCommandDispatcher)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mDocument)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mDocument)
   Updater* updater = tmp->mUpdaters;
   while (updater) {
     cb.NoteXPCOMChild(updater->mElement);
@@ -127,44 +95,44 @@ nsXULCommandDispatcher::Disconnect()
     mUpdaters = mUpdaters->mNext;
     delete doomed;
   }
-  mDocument = nsnull;
+  mDocument = nullptr;
 }
 
 already_AddRefed<nsPIWindowRoot>
 nsXULCommandDispatcher::GetWindowRoot()
 {
   if (mDocument) {
-    nsCOMPtr<nsPIDOMWindow> window(do_QueryInterface(mDocument->GetScriptGlobalObject()));
+    nsCOMPtr<nsPIDOMWindow> window(mDocument->GetWindow());
     if (window) {
       return window->GetTopWindowRoot();
     }
   }
 
-  return nsnull;
+  return nullptr;
 }
 
 nsIContent*
 nsXULCommandDispatcher::GetRootFocusedContentAndWindow(nsPIDOMWindow** aWindow)
 {
-  *aWindow = nsnull;
+  *aWindow = nullptr;
 
   if (mDocument) {
     nsCOMPtr<nsPIDOMWindow> win = mDocument->GetWindow();
     if (win) {
       nsCOMPtr<nsPIDOMWindow> rootWindow = win->GetPrivateRoot();
       if (rootWindow) {
-        return nsFocusManager::GetFocusedDescendant(rootWindow, PR_TRUE, aWindow);
+        return nsFocusManager::GetFocusedDescendant(rootWindow, true, aWindow);
       }
     }
   }
 
-  return nsnull;
+  return nullptr;
 }
 
 NS_IMETHODIMP
 nsXULCommandDispatcher::GetFocusedElement(nsIDOMElement** aElement)
 {
-  *aElement = nsnull;
+  *aElement = nullptr;
 
   nsCOMPtr<nsPIDOMWindow> focusedWindow;
   nsIContent* focusedContent =
@@ -188,7 +156,7 @@ nsXULCommandDispatcher::GetFocusedElement(nsIDOMElement** aElement)
 NS_IMETHODIMP
 nsXULCommandDispatcher::GetFocusedWindow(nsIDOMWindow** aWindow)
 {
-  *aWindow = nsnull;
+  *aWindow = nullptr;
 
   nsCOMPtr<nsPIDOMWindow> window;
   GetRootFocusedContentAndWindow(getter_AddRefs(window));
@@ -240,7 +208,8 @@ nsXULCommandDispatcher::SetFocusedWindow(nsIDOMWindow* aWindow)
   // end up focusing whatever is currently focused inside the frame. Since
   // setting the command dispatcher's focused window doesn't raise the window,
   // setting it to a top-level window doesn't need to do anything.
-  nsCOMPtr<nsIDOMElement> frameElement = window->GetFrameElementInternal();
+  nsCOMPtr<nsIDOMElement> frameElement =
+    do_QueryInterface(window->GetFrameElementInternal());
   if (frameElement)
     return fm->SetFocus(frameElement, 0);
 
@@ -250,7 +219,7 @@ nsXULCommandDispatcher::SetFocusedWindow(nsIDOMWindow* aWindow)
 NS_IMETHODIMP
 nsXULCommandDispatcher::AdvanceFocus()
 {
-  return AdvanceFocusIntoSubtree(nsnull);
+  return AdvanceFocusIntoSubtree(nullptr);
 }
 
 NS_IMETHODIMP
@@ -262,7 +231,7 @@ nsXULCommandDispatcher::RewindFocus()
   nsCOMPtr<nsIDOMElement> result;
   nsIFocusManager* fm = nsFocusManager::GetFocusManager();
   if (fm)
-    return fm->MoveFocus(win, nsnull, nsIFocusManager::MOVEFOCUS_BACKWARD,
+    return fm->MoveFocus(win, nullptr, nsIFocusManager::MOVEFOCUS_BACKWARD,
                          0, getter_AddRefs(result));
   return NS_OK;
 }
@@ -286,10 +255,12 @@ nsXULCommandDispatcher::AddCommandUpdater(nsIDOMElement* aElement,
                                           const nsAString& aEvents,
                                           const nsAString& aTargets)
 {
-  NS_PRECONDITION(aElement != nsnull, "null ptr");
+  NS_PRECONDITION(aElement != nullptr, "null ptr");
   if (! aElement)
     return NS_ERROR_NULL_POINTER;
 
+  NS_ENSURE_TRUE(mDocument, NS_ERROR_UNEXPECTED);
+
   nsresult rv = nsContentUtils::CheckSameOrigin(mDocument, aElement);
 
   if (NS_FAILED(rv)) {
@@ -302,14 +273,14 @@ nsXULCommandDispatcher::AddCommandUpdater(nsIDOMElement* aElement,
   while (updater) {
     if (updater->mElement == aElement) {
 
-#ifdef NS_DEBUG
-      if (PR_LOG_TEST(gLog, PR_LOG_NOTICE)) {
-        nsCAutoString eventsC, targetsC, aeventsC, atargetsC; 
+#ifdef DEBUG
+      if (PR_LOG_TEST(gCommandLog, PR_LOG_NOTICE)) {
+        nsAutoCString eventsC, targetsC, aeventsC, atargetsC; 
         eventsC.AssignWithConversion(updater->mEvents);
         targetsC.AssignWithConversion(updater->mTargets);
         CopyUTF16toUTF8(aEvents, aeventsC);
         CopyUTF16toUTF8(aTargets, atargetsC);
-        PR_LOG(gLog, PR_LOG_NOTICE,
+        PR_LOG(gCommandLog, PR_LOG_NOTICE,
                ("xulcmd[%p] replace %p(events=%s targets=%s) with (events=%s targets=%s)",
                 this, aElement,
                 eventsC.get(),
@@ -330,13 +301,13 @@ nsXULCommandDispatcher::AddCommandUpdater(nsIDOMElement* aElement,
     link = &(updater->mNext);
     updater = updater->mNext;
   }
-#ifdef NS_DEBUG
-  if (PR_LOG_TEST(gLog, PR_LOG_NOTICE)) {
-    nsCAutoString aeventsC, atargetsC; 
+#ifdef DEBUG
+  if (PR_LOG_TEST(gCommandLog, PR_LOG_NOTICE)) {
+    nsAutoCString aeventsC, atargetsC; 
     CopyUTF16toUTF8(aEvents, aeventsC);
     CopyUTF16toUTF8(aTargets, atargetsC);
 
-    PR_LOG(gLog, PR_LOG_NOTICE,
+    PR_LOG(gCommandLog, PR_LOG_NOTICE,
            ("xulcmd[%p] add     %p(events=%s targets=%s)",
             this, aElement,
             aeventsC.get(),
@@ -356,7 +327,7 @@ nsXULCommandDispatcher::AddCommandUpdater(nsIDOMElement* aElement,
 NS_IMETHODIMP
 nsXULCommandDispatcher::RemoveCommandUpdater(nsIDOMElement* aElement)
 {
-  NS_PRECONDITION(aElement != nsnull, "null ptr");
+  NS_PRECONDITION(aElement != nullptr, "null ptr");
   if (! aElement)
     return NS_ERROR_NULL_POINTER;
 
@@ -365,12 +336,12 @@ nsXULCommandDispatcher::RemoveCommandUpdater(nsIDOMElement* aElement)
 
   while (updater) {
     if (updater->mElement == aElement) {
-#ifdef NS_DEBUG
-      if (PR_LOG_TEST(gLog, PR_LOG_NOTICE)) {
-        nsCAutoString eventsC, targetsC; 
+#ifdef DEBUG
+      if (PR_LOG_TEST(gCommandLog, PR_LOG_NOTICE)) {
+        nsAutoCString eventsC, targetsC; 
         eventsC.AssignWithConversion(updater->mEvents);
         targetsC.AssignWithConversion(updater->mTargets);
-        PR_LOG(gLog, PR_LOG_NOTICE,
+        PR_LOG(gCommandLog, PR_LOG_NOTICE,
                ("xulcmd[%p] remove  %p(events=%s targets=%s)",
                 this, aElement,
                 eventsC.get(),
@@ -405,7 +376,7 @@ nsXULCommandDispatcher::UpdateCommands(const nsAString& aEventName)
 
   nsCOMArray<nsIContent> updaters;
 
-  for (Updater* updater = mUpdaters; updater != nsnull; updater = updater->mNext) {
+  for (Updater* updater = mUpdaters; updater != nullptr; updater = updater->mNext) {
     // Skip any nodes that don't match our 'events' or 'targets'
     // filters.
     if (! Matches(updater->mEvents, aEventName))
@@ -415,27 +386,27 @@ nsXULCommandDispatcher::UpdateCommands(const nsAString& aEventName)
       continue;
 
     nsCOMPtr<nsIContent> content = do_QueryInterface(updater->mElement);
-    NS_ASSERTION(content != nsnull, "not an nsIContent");
+    NS_ASSERTION(content != nullptr, "not an nsIContent");
     if (! content)
       return NS_ERROR_UNEXPECTED;
 
     updaters.AppendObject(content);
   }
 
-  for (PRInt32 u = 0; u < updaters.Count(); u++) {
+  for (int32_t u = 0; u < updaters.Count(); u++) {
     nsIContent* content = updaters[u];
 
     nsCOMPtr<nsIDocument> document = content->GetDocument();
 
-    NS_ASSERTION(document != nsnull, "element has no document");
+    NS_ASSERTION(document != nullptr, "element has no document");
     if (! document)
       continue;
 
-#ifdef NS_DEBUG
-    if (PR_LOG_TEST(gLog, PR_LOG_NOTICE)) {
-      nsCAutoString aeventnameC; 
+#ifdef DEBUG
+    if (PR_LOG_TEST(gCommandLog, PR_LOG_NOTICE)) {
+      nsAutoCString aeventnameC; 
       CopyUTF16toUTF8(aEventName, aeventnameC);
-      PR_LOG(gLog, PR_LOG_NOTICE,
+      PR_LOG(gCommandLog, PR_LOG_NOTICE,
              ("xulcmd[%p] update %p event=%s",
               this, content,
               aeventnameC.get()));
@@ -450,40 +421,40 @@ nsXULCommandDispatcher::UpdateCommands(const nsAString& aEventName)
       // Handle the DOM event
       nsEventStatus status = nsEventStatus_eIgnore;
 
-      nsEvent event(PR_TRUE, NS_XUL_COMMAND_UPDATE);
+      WidgetEvent event(true, NS_XUL_COMMAND_UPDATE);
 
-      nsEventDispatcher::Dispatch(content, context, &event, nsnull, &status);
+      EventDispatcher::Dispatch(content, context, &event, nullptr, &status);
     }
   }
   return NS_OK;
 }
 
-PRBool
+bool
 nsXULCommandDispatcher::Matches(const nsString& aList, 
                                 const nsAString& aElement)
 {
   if (aList.EqualsLiteral("*"))
-    return PR_TRUE; // match _everything_!
+    return true; // match _everything_!
 
-  PRInt32 indx = aList.Find(PromiseFlatString(aElement));
+  int32_t indx = aList.Find(PromiseFlatString(aElement));
   if (indx == -1)
-    return PR_FALSE; // not in the list at all
+    return false; // not in the list at all
 
   // okay, now make sure it's not a substring snafu; e.g., 'ur'
   // found inside of 'blur'.
   if (indx > 0) {
-    PRUnichar ch = aList[indx - 1];
-    if (! nsCRT::IsAsciiSpace(ch) && ch != PRUnichar(','))
-      return PR_FALSE;
+    char16_t ch = aList[indx - 1];
+    if (! nsCRT::IsAsciiSpace(ch) && ch != char16_t(','))
+      return false;
   }
 
   if (indx + aElement.Length() < aList.Length()) {
-    PRUnichar ch = aList[indx + aElement.Length()];
-    if (! nsCRT::IsAsciiSpace(ch) && ch != PRUnichar(','))
-      return PR_FALSE;
+    char16_t ch = aList[indx + aElement.Length()];
+    if (! nsCRT::IsAsciiSpace(ch) && ch != char16_t(','))
+      return false;
   }
 
-  return PR_TRUE;
+  return true;
 }
 
 NS_IMETHODIMP
@@ -505,14 +476,14 @@ nsXULCommandDispatcher::GetControllerForCommand(const char *aCommand, nsIControl
 }
 
 NS_IMETHODIMP
-nsXULCommandDispatcher::GetSuppressFocusScroll(PRBool* aSuppressFocusScroll)
+nsXULCommandDispatcher::GetSuppressFocusScroll(bool* aSuppressFocusScroll)
 {
-  *aSuppressFocusScroll = PR_FALSE;
+  *aSuppressFocusScroll = false;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsXULCommandDispatcher::SetSuppressFocusScroll(PRBool aSuppressFocusScroll)
+nsXULCommandDispatcher::SetSuppressFocusScroll(bool aSuppressFocusScroll)
 {
   return NS_OK;
 }
