@@ -17,17 +17,18 @@ public:
     TestSysVShmemParent() { }
     virtual ~TestSysVShmemParent() { }
 
+    static bool RunTestInProcesses() { return true; }
+    static bool RunTestInThreads() { return true; }
+
     void Main();
 
 protected:
-    NS_OVERRIDE
     virtual bool RecvTake(
             Shmem& mem,
             Shmem& unsafe,
-            const size_t& expectedSize);
+            const size_t& expectedSize) MOZ_OVERRIDE;
 
-    NS_OVERRIDE
-    virtual void ActorDestroy(ActorDestroyReason why)
+    virtual void ActorDestroy(ActorDestroyReason why) MOZ_OVERRIDE
     {
         if (NormalShutdown != why)
             fail("unexpected destruction!");  
@@ -45,14 +46,12 @@ public:
     virtual ~TestSysVShmemChild() { }
 
 protected:
-    NS_OVERRIDE
     virtual bool RecvGive(
             Shmem& mem,
             Shmem& unsafe,
-            const size_t& expectedSize);
+            const size_t& expectedSize) MOZ_OVERRIDE;
 
-    NS_OVERRIDE
-    virtual void ActorDestroy(ActorDestroyReason why)
+    virtual void ActorDestroy(ActorDestroyReason why) MOZ_OVERRIDE
     {
         if (NormalShutdown != why)
             fail("unexpected destruction!");
