@@ -4,15 +4,18 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch10/10.1/10.1.1/10.1.1-9-s.js
- * @description Strict Mode - Use Strict Directive Prologue is ''Use strict';' in which the first character is uppercase
+ * @path ch10/10.4/10.4.3/10.4.3-1-47-s.js
+ * @description Strict Mode - checking 'this' (Anonymous FunctionExpression with a strict directive prologue defined within a FunctionDeclaration)
  * @noStrict
  */
-
-
+    
 function testcase() {
-        "Use strict";
-        var public = 1;
-        return public === 1;
-    }
-runTestCase(testcase);
+function f1() {
+    return ((function () {
+        "use strict";
+        return typeof this;
+    })()==="undefined") && (this===fnGlobalObject());
+}
+return f1();
+}
+runTestCase(testcase);
\ No newline at end of file
