@@ -4,19 +4,20 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch11/11.13/11.13.2/11.13.2-6-s.js
- * @description Strict Mode - ReferenceError is thrown if the LeftHandSideExpression of a Compound Assignment operator(<<=) evaluates to an unresolvable reference
+ * @path ch11/11.4/11.4.1/11.4.1-5-a-28-s.js
+ * @description Strict Mode - TypeError is thrown when deleting RegExp.length
  * @onlyStrict
  */
 
 
 function testcase() {
-        "use strict";
-        try {
-            eval("_11_13_2_6 <<= 1;");
-            return false;
-        } catch (e) {
-            return e instanceof ReferenceError;
-        }
+    "use strict";
+    var a = new RegExp();
+    try {
+        var b = delete RegExp.length;
+        return false;
+    } catch (e) {
+        return e instanceof TypeError;
     }
+}
 runTestCase(testcase);
