@@ -1,7 +1,6 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
- * vim: set ts=8 sw=4 et tw=99 ft=cpp:
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
+ * vim: set ts=8 sts=4 et sw=4 tw=99:
  *
- * ***** BEGIN LICENSE BLOCK *****
  * Copyright (C) 2009 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -23,19 +22,16 @@
  * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
  * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
- *
- * ***** END LICENSE BLOCK ***** */
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ */
 
-#ifndef YarrParser_h
-#define YarrParser_h
+#ifndef yarr_YarrParser_h
+#define yarr_YarrParser_h
 
-#include "Yarr.h"
+#include "yarr/Yarr.h"
 
 namespace JSC { namespace Yarr {
 
-#define REGEXP_ERROR_PREFIX "Invalid regular expression: "
-
 enum BuiltInCharacterClassID {
     DigitClassID,
     SpaceClassID,
@@ -44,11 +40,11 @@ enum BuiltInCharacterClassID {
 };
 
 // The Parser class should not be used directly - only via the Yarr::parse() method.
-template<class Delegate>
+template<class Delegate, typename CharType>
 class Parser {
 private:
     template<class FriendDelegate>
-    friend ErrorCode parse(FriendDelegate& delegate, const UString& pattern, unsigned backReferenceLimit);
+    friend ErrorCode parse(FriendDelegate&, const String& pattern, unsigned backReferenceLimit);
 
     /*
      * CharacterClassParserDelegate:
@@ -159,7 +155,6 @@ private:
                 // the end of the range to be a single character.
                 m_err = CharacterClassInvalidRange;
                 return;
-
             case AfterCharacterClassHyphen:
                 m_delegate.atomCharacterClassBuiltIn(classID, invert);
                 m_state = Empty;
@@ -185,8 +180,8 @@ private:
 
         // parseEscape() should never call these delegate methods when
         // invoked with inCharacterClass set.
-        void assertionWordBoundary(bool) { ASSERT_NOT_REACHED(); }
-        void atomBackReference(unsigned) { ASSERT_NOT_REACHED(); }
+        NO_RETURN_DUE_TO_ASSERT void assertionWordBoundary(bool) { ASSERT_NOT_REACHED(); }
+        NO_RETURN_DUE_TO_ASSERT void atomBackReference(unsigned) { ASSERT_NOT_REACHED(); }
 
     private:
         Delegate& m_delegate;
@@ -201,7 +196,7 @@ private:
         UChar m_character;
     };
 
-    Parser(Delegate& delegate, const UString& pattern, unsigned backReferenceLimit)
+    Parser(Delegate& delegate, const String& pattern, unsigned backReferenceLimit)
         : m_delegate(delegate)
         , m_backReferenceLimit(backReferenceLimit)
         , m_err(NoError)
@@ -211,7 +206,7 @@ private:
         , m_parenthesesNestingDepth(0)
     {
     }
-    
+
     /*
      * parseEscape():
      *
@@ -301,7 +296,7 @@ private:
         case '7':
         case '8':
         case '9': {
-            // To match Firefox, we parse an invalid backreference in the range [1-7] as an octal escape.
+            // To match IceCat, we parse an invalid backreference in the range [1-7] as an octal escape.
             // First, try to parse this as backreference.
             if (!inCharacterClass) {
                 ParseState state = saveState();
@@ -316,7 +311,7 @@ private:
 
                 restoreState(state);
             }
-            
+
             // Not a backreference, and not octal.
             if (peek() >= '8') {
                 delegate.atomPatternCharacter('\\');
@@ -360,7 +355,7 @@ private:
             if (!atEndOfPattern()) {
                 int control = consume();
 
-                // To match Firefox, inside a character class, we also accept numbers and '_' as control characters.
+                // To match IceCat, inside a character class, we also accept numbers and '_' as control characters.
                 if (inCharacterClass ? WTF::isASCIIAlphanumeric(control) || (control == '_') : WTF::isASCIIAlpha(control)) {
                     delegate.atomPatternCharacter(control & 0x1f);
                     break;
@@ -397,7 +392,7 @@ private:
         default:
             delegate.atomPatternCharacter(consume());
         }
-        
+
         return true;
     }
 
@@ -475,7 +470,7 @@ private:
             case ':':
                 m_delegate.atomParenthesesSubpatternBegin(false);
                 break;
-            
+
             case '=':
                 m_delegate.atomParentheticalAssertionBegin();
                 break;
@@ -483,7 +478,7 @@ private:
             case '!':
                 m_delegate.atomParentheticalAssertionBegin(true);
                 break;
-            
+
             default:
                 m_err = ParenthesesTypeInvalid;
             }
@@ -522,6 +517,11 @@ private:
         ASSERT(!m_err);
         ASSERT(min <= max);
 
+        if (min == UINT_MAX) {
+            m_err = QuantifierTooLarge;
+            return;
+        }
+
         if (lastTokenWasAnAtom)
             m_delegate.quantifyAtom(min, max, !tryConsume('?'));
         else
@@ -612,8 +612,8 @@ private:
                     unsigned min;
                     if (!consumeNumber(min))
                         break;
+
                     unsigned max = min;
-                    
                     if (tryConsume(',')) {
                         if (peekIsDigit()) {
                             if (!consumeNumber(max))
@@ -666,11 +666,10 @@ private:
         return m_err;
     }
 
-
     // Misc helper functions:
 
     typedef unsigned ParseState;
-    
+
     ParseState saveState()
     {
         return m_index;
@@ -767,7 +766,7 @@ private:
     Delegate& m_delegate;
     unsigned m_backReferenceLimit;
     ErrorCode m_err;
-    const UChar* m_data;
+    const CharType* m_data;
     unsigned m_size;
     unsigned m_index;
     unsigned m_parenthesesNestingDepth;
@@ -836,11 +835,15 @@ private:
  */
 
 template<class Delegate>
-ErrorCode parse(Delegate& delegate, const UString& pattern, unsigned backReferenceLimit = quantifyInfinite)
+ErrorCode parse(Delegate& delegate, const String& pattern, unsigned backReferenceLimit = quantifyInfinite)
 {
-    return Parser<Delegate>(delegate, pattern, backReferenceLimit).parse();
+#ifdef YARR_8BIT_CHAR_SUPPORT
+    if (pattern.is8Bit())
+        return Parser<Delegate, LChar>(delegate, pattern, backReferenceLimit).parse();
+#endif
+    return Parser<Delegate, UChar>(delegate, pattern, backReferenceLimit).parse();
 }
 
 } } // namespace JSC::Yarr
 
-#endif // YarrParser_h
+#endif /* yarr_YarrParser_h */
