@@ -1,47 +1,14 @@
 /* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
  *
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * IBM Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2000
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Simon Montagu
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
-#ifdef IBMBIDI
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "prmem.h"
 #include "nsBidi.h"
-#include "nsBidiUtils.h"
-#include "nsCRT.h"
+#include "nsUnicodeProperties.h"
+#include "nsCRTGlue.h"
+
+using namespace mozilla::unicode;
 
 // These are #defined in <sys/regset.h> under Solaris 10 x86
 #undef CS
@@ -166,8 +133,8 @@ nsBidi::nsBidi()
 {
   Init();
 
-  mMayAllocateText=PR_TRUE;
-  mMayAllocateRuns=PR_TRUE;
+  mMayAllocateText=true;
+  mMayAllocateRuns=true;
 }
 
 nsBidi::~nsBidi()
@@ -177,7 +144,7 @@ nsBidi::~nsBidi()
 
 void nsBidi::Init()
 {
-  /* reset the object, all pointers NULL, all flags PR_FALSE, all sizes 0 */
+  /* reset the object, all pointers nullptr, all flags false, all sizes 0 */
   mLength = 0;
   mParaLevel = 0;
   mFlags = 0;
@@ -189,89 +156,92 @@ void nsBidi::Init()
   mRunsSize = 0;
   mRunCount = -1;
 
-  mDirProps=NULL;
-  mLevels=NULL;
-  mRuns=NULL;
+  mDirProps=nullptr;
+  mLevels=nullptr;
+  mRuns=nullptr;
 
-  mDirPropsMemory=NULL;
-  mLevelsMemory=NULL;
-  mRunsMemory=NULL;
+  mDirPropsMemory=nullptr;
+  mLevelsMemory=nullptr;
+  mRunsMemory=nullptr;
 
-  mMayAllocateText=PR_FALSE;
-  mMayAllocateRuns=PR_FALSE;
+  mMayAllocateText=false;
+  mMayAllocateRuns=false;
   
 }
 
 /*
- * We are allowed to allocate memory if aMemory==NULL or
- * aMayAllocate==PR_TRUE for each array that we need.
+ * We are allowed to allocate memory if aMemory==nullptr or
+ * aMayAllocate==true for each array that we need.
  * We also try to grow and shrink memory as needed if we
  * allocate it.
  *
  * Assume aSizeNeeded>0.
- * If *aMemory!=NULL, then assume *aSize>0.
+ * If *aMemory!=nullptr, then assume *aSize>0.
  *
  * ### this realloc() may unnecessarily copy the old data,
  * which we know we don't need any more;
  * is this the best way to do this??
  */
-PRBool nsBidi::GetMemory(void **aMemory, PRSize *aSize, PRBool aMayAllocate, PRSize aSizeNeeded)
+bool nsBidi::GetMemory(void **aMemory, size_t *aSize, bool aMayAllocate, size_t aSizeNeeded)
 {
   /* check for existing memory */
-  if(*aMemory==NULL) {
+  if(*aMemory==nullptr) {
     /* we need to allocate memory */
     if(!aMayAllocate) {
-      return PR_FALSE;
+      return false;
     } else {
-      *aMemory=PR_MALLOC(aSizeNeeded);
-      if (*aMemory!=NULL) {
+      *aMemory=moz_malloc(aSizeNeeded);
+      if (*aMemory!=nullptr) {
         *aSize=aSizeNeeded;
-        return PR_TRUE;
+        return true;
       } else {
         *aSize=0;
-        return PR_FALSE;
+        return false;
       }
     }
   } else {
     /* there is some memory, is it enough or too much? */
     if(aSizeNeeded>*aSize && !aMayAllocate) {
       /* not enough memory, and we must not allocate */
-      return PR_FALSE;
+      return false;
     } else if(aSizeNeeded!=*aSize && aMayAllocate) {
       /* we may try to grow or shrink */
-      void *memory=PR_REALLOC(*aMemory, aSizeNeeded);
+      void *memory=moz_realloc(*aMemory, aSizeNeeded);
 
-      if(memory!=NULL) {
+      if(memory!=nullptr) {
         *aMemory=memory;
         *aSize=aSizeNeeded;
-        return PR_TRUE;
+        return true;
       } else {
         /* we failed to grow */
-        return PR_FALSE;
+        return false;
       }
     } else {
       /* we have at least enough memory and must not allocate */
-      return PR_TRUE;
+      return true;
     }
   }
 }
 
 void nsBidi::Free()
 {
-  PR_FREEIF(mDirPropsMemory);
-  PR_FREEIF(mLevelsMemory);
-  PR_FREEIF(mRunsMemory);
+  moz_free(mDirPropsMemory);
+  mDirPropsMemory = nullptr;
+  moz_free(mLevelsMemory);
+  mLevelsMemory = nullptr;
+  moz_free(mRunsMemory);
+  mRunsMemory = nullptr;
 }
 
 /* SetPara ------------------------------------------------------------ */
 
-nsresult nsBidi::SetPara(const PRUnichar *aText, PRInt32 aLength,
+nsresult nsBidi::SetPara(const char16_t *aText, int32_t aLength,
                          nsBidiLevel aParaLevel, nsBidiLevel *aEmbeddingLevels)
 {
   nsBidiDirection direction;
 
   /* check the argument values */
-  if(aText==NULL ||
+  if(aText==nullptr ||
      ((NSBIDI_MAX_EXPLICIT_LEVEL<aParaLevel) && !IS_DEFAULT_LEVEL(aParaLevel)) ||
      aLength<-1
     ) {
@@ -279,7 +249,7 @@ nsresult nsBidi::SetPara(const PRUnichar *aText, PRInt32 aLength,
   }
 
   if(aLength==-1) {
-    aLength=nsCRT::strlen(aText);
+    aLength = NS_strlen(aText);
   }
 
   /* initialize member data */
@@ -288,9 +258,9 @@ nsresult nsBidi::SetPara(const PRUnichar *aText, PRInt32 aLength,
   mDirection=NSBIDI_LTR;
   mTrailingWSStart=aLength;  /* the levels[] will reflect the WS run */
 
-  mDirProps=NULL;
-  mLevels=NULL;
-  mRuns=NULL;
+  mDirProps=nullptr;
+  mLevels=nullptr;
+  mRuns=nullptr;
 
   if(aLength==0) {
     /*
@@ -328,7 +298,7 @@ nsresult nsBidi::SetPara(const PRUnichar *aText, PRInt32 aLength,
   }
 
   /* are explicit levels specified? */
-  if(aEmbeddingLevels==NULL) {
+  if(aEmbeddingLevels==nullptr) {
     /* no: determine explicit levels according to the (Xn) rules */\
     if(GETLEVELSMEMORY(aLength)) {
       mLevels=mLevelsMemory;
@@ -376,14 +346,14 @@ nsresult nsBidi::SetPara(const PRUnichar *aText, PRInt32 aLength,
        * Examples for "insignificant" ones are empty embeddings
        * LRE-PDF, LRE-RLE-PDF-PDF, etc.
        */
-      if(aEmbeddingLevels==NULL && !(mFlags&DIRPROP_FLAG_MULTI_RUNS)) {
+      if(aEmbeddingLevels==nullptr && !(mFlags&DIRPROP_FLAG_MULTI_RUNS)) {
         ResolveImplicitLevels(0, aLength,
                     GET_LR_FROM_LEVEL(mParaLevel),
                     GET_LR_FROM_LEVEL(mParaLevel));
       } else {
         /* sor, eor: start and end types of same-level-run */
         nsBidiLevel *levels=mLevels;
-        PRInt32 start, limit=0;
+        int32_t start, limit=0;
         nsBidiLevel level, nextLevel;
         DirProp sor, eor;
 
@@ -445,13 +415,13 @@ nsresult nsBidi::SetPara(const PRUnichar *aText, PRInt32 aLength,
  * calculate the flags bit-set, and
  * determine the partagraph level if necessary.
  */
-void nsBidi::GetDirProps(const PRUnichar *aText)
+void nsBidi::GetDirProps(const char16_t *aText)
 {
   DirProp *dirProps=mDirPropsMemory;    /* mDirProps is const */
 
-  PRInt32 i=0, length=mLength;
+  int32_t i=0, length=mLength;
   Flags flags=0;      /* collect all directionalities in the text */
-  PRUnichar uchar;
+  char16_t uchar;
   DirProp dirProp;
 
   if(IS_DEFAULT_LEVEL(mParaLevel)) {
@@ -460,11 +430,11 @@ void nsBidi::GetDirProps(const PRUnichar *aText)
       uchar=aText[i];
       if(!IS_FIRST_SURROGATE(uchar) || i+1==length || !IS_SECOND_SURROGATE(aText[i+1])) {
         /* not a surrogate pair */
-        flags|=DIRPROP_FLAG(dirProps[i]=dirProp=GetCharType((PRUint32)uchar));
+        flags|=DIRPROP_FLAG(dirProps[i]=dirProp=GetBidiCat((uint32_t)uchar));
       } else {
         /* a surrogate pair */
         dirProps[i++]=BN;   /* first surrogate in the pair gets the BN type */
-        flags|=DIRPROP_FLAG(dirProps[i]=dirProp=GetCharType(GET_UTF_32(uchar, aText[i])))|DIRPROP_FLAG(BN);
+        flags|=DIRPROP_FLAG(dirProps[i]=dirProp=GetBidiCat(GET_UTF_32(uchar, aText[i])))|DIRPROP_FLAG(BN);
       }
       ++i;
       if(dirProp==L) {
@@ -490,11 +460,11 @@ void nsBidi::GetDirProps(const PRUnichar *aText)
     uchar=aText[i];
     if(!IS_FIRST_SURROGATE(uchar) || i+1==length || !IS_SECOND_SURROGATE(aText[i+1])) {
       /* not a surrogate pair */
-      flags|=DIRPROP_FLAG(dirProps[i]=GetCharType((PRUint32)uchar));
+      flags|=DIRPROP_FLAG(dirProps[i]=GetBidiCat((uint32_t)uchar));
     } else {
       /* a surrogate pair */
       dirProps[i++]=BN;   /* second surrogate in the pair gets the BN type */
-      flags|=DIRPROP_FLAG(dirProps[i]=GetCharType(GET_UTF_32(uchar, aText[i])))|DIRPROP_FLAG(BN);
+      flags|=DIRPROP_FLAG(dirProps[i]=GetBidiCat(GET_UTF_32(uchar, aText[i])))|DIRPROP_FLAG(BN);
     }
     ++i;
   }
@@ -564,7 +534,7 @@ nsBidiDirection nsBidi::ResolveExplicitLevels()
   const DirProp *dirProps=mDirProps;
   nsBidiLevel *levels=mLevels;
 
-  PRInt32 i=0, length=mLength;
+  int32_t i=0, length=mLength;
   Flags flags=mFlags;       /* collect all directionalities in the text */
   DirProp dirProp;
   nsBidiLevel level=mParaLevel;
@@ -591,7 +561,7 @@ nsBidiDirection nsBidi::ResolveExplicitLevels()
     nsBidiLevel embeddingLevel=level, newLevel, stackTop=0;
 
     nsBidiLevel stack[NSBIDI_MAX_EXPLICIT_LEVEL];        /* we never push anything >=NSBIDI_MAX_EXPLICIT_LEVEL */
-    PRUint32 countOver60=0, countOver61=0;  /* count overflows of explicit levels */
+    uint32_t countOver60=0, countOver61=0;  /* count overflows of explicit levels */
 
     /* recalculate the flags */
     flags=0;
@@ -720,7 +690,7 @@ nsresult nsBidi::CheckExplicitLevels(nsBidiDirection *aDirection)
   const DirProp *dirProps=mDirProps;
   nsBidiLevel *levels=mLevels;
 
-  PRInt32 i, length=mLength;
+  int32_t i, length=mLength;
   Flags flags=0;  /* collect all directionalities in the text */
   nsBidiLevel level, paraLevel=mParaLevel;
 
@@ -795,15 +765,15 @@ nsBidiDirection nsBidi::DirectionFromFlags(Flags aFlags)
 #define PREV_EN_AFTER_W2 4
 #define PREV_EN_AFTER_W4 8
 
-void nsBidi::ResolveImplicitLevels(PRInt32 aStart, PRInt32 aLimit,
+void nsBidi::ResolveImplicitLevels(int32_t aStart, int32_t aLimit,
                    DirProp aSOR, DirProp aEOR)
 {
   const DirProp *dirProps=mDirProps;
   nsBidiLevel *levels=mLevels;
 
-  PRInt32 i, next, neutralStart=-1;
+  int32_t i, next, neutralStart=-1;
   DirProp prevDirProp, dirProp, nextDirProp, lastStrong, beforeNeutral;
-  PRUint8 historyOfEN;
+  uint8_t historyOfEN;
 
   /* initialize: current at aSOR, next at aStart (it is aStart<aLimit) */
   next=aStart;
@@ -1081,7 +1051,7 @@ void nsBidi::AdjustWSLevels()
 {
   const DirProp *dirProps=mDirProps;
   nsBidiLevel *levels=mLevels;
-  PRInt32 i;
+  int32_t i;
 
   if(mFlags&MASK_WS) {
     nsBidiLevel paraLevel=mParaLevel;
@@ -1122,21 +1092,21 @@ nsresult nsBidi::GetDirection(nsBidiDirection* aDirection)
   *aDirection = mDirection;
   return NS_OK;
 }
-#ifdef FULL_BIDI_ENGINE
-
-/* -------------------------------------------------------------------------- */
-
-nsresult nsBidi::GetLength(PRInt32* aLength)
-{
-  *aLength = mLength;
-  return NS_OK;
-}
 
 nsresult nsBidi::GetParaLevel(nsBidiLevel* aParaLevel)
 {
   *aParaLevel = mParaLevel;
   return NS_OK;
 }
+#ifdef FULL_BIDI_ENGINE
+
+/* -------------------------------------------------------------------------- */
+
+nsresult nsBidi::GetLength(int32_t* aLength)
+{
+  *aLength = mLength;
+  return NS_OK;
+}
 
 /*
  * General remarks about the functions in this section:
@@ -1151,7 +1121,7 @@ nsresult nsBidi::GetParaLevel(nsBidiLevel* aParaLevel)
  * This means that there is a nsBidi object with a levels
  * and a dirProps array.
  * paraLevel and direction are also set.
- * Only if the length of the text is zero, then levels==dirProps==NULL.
+ * Only if the length of the text is zero, then levels==dirProps==nullptr.
  *
  * The overall directionality of the paragraph
  * or line is used to bypass the reordering steps if possible.
@@ -1187,13 +1157,13 @@ nsresult nsBidi::GetParaLevel(nsBidiLevel* aParaLevel)
  * a pointer into them, not by copying. This again saves memory and forbids to
  * change the now shared levels for (L1).
  */
-nsresult nsBidi::SetLine(nsIBidi* aParaBidi, PRInt32 aStart, PRInt32 aLimit)
+nsresult nsBidi::SetLine(nsIBidi* aParaBidi, int32_t aStart, int32_t aLimit)
 {
   nsBidi* pParent = (nsBidi*)aParaBidi;
-  PRInt32 length;
+  int32_t length;
 
   /* check the argument values */
-  if(pParent==NULL) {
+  if(pParent==nullptr) {
     return NS_ERROR_INVALID_POINTER;
   } else if(aStart<0 || aStart>aLimit || aLimit>pParent->mLength) {
     return NS_ERROR_INVALID_ARG;
@@ -1203,7 +1173,7 @@ nsresult nsBidi::SetLine(nsIBidi* aParaBidi, PRInt32 aStart, PRInt32 aLimit)
   length=mLength=aLimit-aStart;
   mParaLevel=pParent->mParaLevel;
 
-  mRuns=NULL;
+  mRuns=nullptr;
   mFlags=0;
 
   if(length>0) {
@@ -1229,7 +1199,7 @@ nsresult nsBidi::SetLine(nsIBidi* aParaBidi, PRInt32 aStart, PRInt32 aLimit)
       }
     } else {
       const nsBidiLevel *levels=mLevels;
-      PRInt32 i, trailingWSStart;
+      int32_t i, trailingWSStart;
       nsBidiLevel level;
       Flags flags=0;
 
@@ -1289,8 +1259,8 @@ nsresult nsBidi::SetLine(nsIBidi* aParaBidi, PRInt32 aStart, PRInt32 aLimit)
     mDirection=mParaLevel&1 ? NSBIDI_RTL : NSBIDI_LTR;
     mTrailingWSStart=mRunCount=0;
 
-    mDirProps=NULL;
-    mLevels=NULL;
+    mDirProps=nullptr;
+    mLevels=nullptr;
   }
   return NS_OK;
 }
@@ -1311,7 +1281,7 @@ void nsBidi::SetTrailingWSStart() {
 
   const DirProp *dirProps=mDirProps;
   nsBidiLevel *levels=mLevels;
-  PRInt32 start=mLength;
+  int32_t start=mLength;
   nsBidiLevel paraLevel=mParaLevel;
 
   /* go backwards across all WS, BN, explicit codes */
@@ -1327,7 +1297,7 @@ void nsBidi::SetTrailingWSStart() {
   mTrailingWSStart=start;
 }
 
-nsresult nsBidi::GetLevelAt(PRInt32 aCharIndex, nsBidiLevel* aLevel)
+nsresult nsBidi::GetLevelAt(int32_t aCharIndex, nsBidiLevel* aLevel)
 {
   /* return paraLevel if in the trailing WS run, otherwise the real level */
   if(aCharIndex<0 || mLength<=aCharIndex) {
@@ -1342,11 +1312,11 @@ nsresult nsBidi::GetLevelAt(PRInt32 aCharIndex, nsBidiLevel* aLevel)
 
 nsresult nsBidi::GetLevels(nsBidiLevel** aLevels)
 {
-  PRInt32 start, length;
+  int32_t start, length;
 
   length = mLength;
   if(length<=0) {
-    *aLevels = NULL;
+    *aLevels = nullptr;
     return NS_ERROR_INVALID_ARG;
   }
 
@@ -1379,13 +1349,13 @@ nsresult nsBidi::GetLevels(nsBidiLevel** aLevels)
     return NS_OK;
   } else {
     /* out of memory */
-    *aLevels = NULL;
+    *aLevels = nullptr;
     return NS_ERROR_OUT_OF_MEMORY;
   }
 }
 #endif // FULL_BIDI_ENGINE
 
-nsresult nsBidi::GetCharTypeAt(PRInt32 aCharIndex, nsCharType* pType)
+nsresult nsBidi::GetCharTypeAt(int32_t aCharIndex, nsCharType* pType)
 {
   if(aCharIndex<0 || mLength<=aCharIndex) {
     return NS_ERROR_INVALID_ARG;
@@ -1394,19 +1364,19 @@ nsresult nsBidi::GetCharTypeAt(PRInt32 aCharIndex, nsCharType* pType)
   return NS_OK;
 }
 
-nsresult nsBidi::GetLogicalRun(PRInt32 aLogicalStart, PRInt32 *aLogicalLimit, nsBidiLevel *aLevel)
+nsresult nsBidi::GetLogicalRun(int32_t aLogicalStart, int32_t *aLogicalLimit, nsBidiLevel *aLevel)
 {
-  PRInt32 length = mLength;
+  int32_t length = mLength;
 
   if(aLogicalStart<0 || length<=aLogicalStart) {
     return NS_ERROR_INVALID_ARG;
   }
 
   if(mDirection!=NSBIDI_MIXED || aLogicalStart>=mTrailingWSStart) {
-    if(aLogicalLimit!=NULL) {
+    if(aLogicalLimit!=nullptr) {
       *aLogicalLimit=length;
     }
-    if(aLevel!=NULL) {
+    if(aLevel!=nullptr) {
       *aLevel=mParaLevel;
     }
   } else {
@@ -1417,10 +1387,10 @@ nsresult nsBidi::GetLogicalRun(PRInt32 aLogicalStart, PRInt32 *aLogicalLimit, ns
     length=mTrailingWSStart;
     while(++aLogicalStart<length && level==levels[aLogicalStart]) {}
 
-    if(aLogicalLimit!=NULL) {
+    if(aLogicalLimit!=nullptr) {
       *aLogicalLimit=aLogicalStart;
     }
-    if(aLevel!=NULL) {
+    if(aLevel!=nullptr) {
       *aLevel=level;
     }
   }
@@ -1429,7 +1399,7 @@ nsresult nsBidi::GetLogicalRun(PRInt32 aLogicalStart, PRInt32 *aLogicalLimit, ns
 
 /* runs API functions ------------------------------------------------------- */
 
-nsresult nsBidi::CountRuns(PRInt32* aRunCount)
+nsresult nsBidi::CountRuns(int32_t* aRunCount)
 {
   if(mRunCount<0 && !GetRuns()) {
     return NS_ERROR_OUT_OF_MEMORY;
@@ -1440,7 +1410,7 @@ nsresult nsBidi::CountRuns(PRInt32* aRunCount)
   }
 }
 
-nsresult nsBidi::GetVisualRun(PRInt32 aRunIndex, PRInt32 *aLogicalStart, PRInt32 *aLength, nsBidiDirection *aDirection)
+nsresult nsBidi::GetVisualRun(int32_t aRunIndex, int32_t *aLogicalStart, int32_t *aLength, nsBidiDirection *aDirection)
 {
   if( aRunIndex<0 ||
       (mRunCount==-1 && !GetRuns()) ||
@@ -1449,11 +1419,11 @@ nsresult nsBidi::GetVisualRun(PRInt32 aRunIndex, PRInt32 *aLogicalStart, PRInt32
     *aDirection = NSBIDI_LTR;
     return NS_OK;
   } else {
-    PRInt32 start=mRuns[aRunIndex].logicalStart;
-    if(aLogicalStart!=NULL) {
+    int32_t start=mRuns[aRunIndex].logicalStart;
+    if(aLogicalStart!=nullptr) {
       *aLogicalStart=GET_INDEX(start);
     }
-    if(aLength!=NULL) {
+    if(aLength!=nullptr) {
       if(aRunIndex>0) {
         *aLength=mRuns[aRunIndex].visualLimit-
              mRuns[aRunIndex-1].visualLimit;
@@ -1470,19 +1440,19 @@ nsresult nsBidi::GetVisualRun(PRInt32 aRunIndex, PRInt32 *aLogicalStart, PRInt32
 
 /*
  * Compute the runs array from the levels array.
- * After GetRuns() returns PR_TRUE, runCount is guaranteed to be >0
+ * After GetRuns() returns true, runCount is guaranteed to be >0
  * and the runs are reordered.
  * Odd-level runs have visualStart on their visual right edge and
  * they progress visually to the left.
  */
-PRBool nsBidi::GetRuns()
+bool nsBidi::GetRuns()
 {
   if(mDirection!=NSBIDI_MIXED) {
     /* simple, single-run case - this covers length==0 */
     GetSingleRun(mParaLevel);
   } else /* NSBIDI_MIXED, length>0 */ {
     /* mixed directionality */
-    PRInt32 length=mLength, limit=length;
+    int32_t length=mLength, limit=length;
 
     /*
      * If there are WS characters at the end of the line
@@ -1501,7 +1471,7 @@ PRBool nsBidi::GetRuns()
       GetSingleRun(mParaLevel);
     } else {
       nsBidiLevel *levels=mLevels;
-      PRInt32 i, runCount;
+      int32_t i, runCount;
       nsBidiLevel level=NSBIDI_DEFAULT_LTR;   /* initialize with no valid level */
 
       /* count the runs, there is at least one non-WS run, and limit>0 */
@@ -1524,7 +1494,7 @@ PRBool nsBidi::GetRuns()
       } else /* runCount>1 || limit<length */ {
         /* allocate and set the runs */
         Run *runs;
-        PRInt32 runIndex, start;
+        int32_t runIndex, start;
         nsBidiLevel minLevel=NSBIDI_MAX_EXPLICIT_LEVEL+1, maxLevel=0;
 
         /* now, count a (non-mergable) WS run */
@@ -1536,7 +1506,7 @@ PRBool nsBidi::GetRuns()
         if(GETRUNSMEMORY(runCount)) {
           runs=mRunsMemory;
         } else {
-          return PR_FALSE;
+          return false;
         }
 
         /* set the runs */
@@ -1609,7 +1579,7 @@ PRBool nsBidi::GetRuns()
       }
     }
   }
-  return PR_TRUE;
+  return true;
 }
 
 /* in trivial cases there is only one trivial run; called by GetRuns() */
@@ -1661,7 +1631,7 @@ void nsBidi::ReorderLine(nsBidiLevel aMinLevel, nsBidiLevel aMaxLevel)
 {
   Run *runs;
   nsBidiLevel *levels;
-  PRInt32 firstRun, endRun, limitRun, runCount, temp;
+  int32_t firstRun, endRun, limitRun, runCount, temp;
 
   /* nothing to do? */
   if(aMaxLevel<=(aMinLevel|1)) {
@@ -1749,12 +1719,13 @@ void nsBidi::ReorderLine(nsBidiLevel aMinLevel, nsBidiLevel aMaxLevel)
   }
 }
 
-nsresult nsBidi::ReorderVisual(const nsBidiLevel *aLevels, PRInt32 aLength, PRInt32 *aIndexMap)
+nsresult nsBidi::ReorderVisual(const nsBidiLevel *aLevels, int32_t aLength, int32_t *aIndexMap)
 {
-  PRInt32 start, end, limit, temp;
+  int32_t start, end, limit, temp;
   nsBidiLevel minLevel, maxLevel;
 
-  if(aIndexMap==NULL || !PrepareReorder(aLevels, aLength, aIndexMap, &minLevel, &maxLevel)) {
+  if(aIndexMap==nullptr ||
+     !PrepareReorder(aLevels, aLength, aIndexMap, &minLevel, &maxLevel)) {
     return NS_OK;
   }
 
@@ -1811,15 +1782,15 @@ nsresult nsBidi::ReorderVisual(const nsBidiLevel *aLevels, PRInt32 aLength, PRIn
   return NS_OK;
 }
 
-PRBool nsBidi::PrepareReorder(const nsBidiLevel *aLevels, PRInt32 aLength,
-                PRInt32 *aIndexMap,
+bool nsBidi::PrepareReorder(const nsBidiLevel *aLevels, int32_t aLength,
+                int32_t *aIndexMap,
                 nsBidiLevel *aMinLevel, nsBidiLevel *aMaxLevel)
 {
-  PRInt32 start;
+  int32_t start;
   nsBidiLevel level, minLevel, maxLevel;
 
-  if(aLevels==NULL || aLength<=0) {
-    return PR_FALSE;
+  if(aLevels==nullptr || aLength<=0) {
+    return false;
   }
 
   /* determine minLevel and maxLevel */
@@ -1828,7 +1799,7 @@ PRBool nsBidi::PrepareReorder(const nsBidiLevel *aLevels, PRInt32 aLength,
   for(start=aLength; start>0;) {
     level=aLevels[--start];
     if(level>NSBIDI_MAX_EXPLICIT_LEVEL+1) {
-      return PR_FALSE;
+      return false;
     }
     if(level<minLevel) {
       minLevel=level;
@@ -1846,13 +1817,13 @@ PRBool nsBidi::PrepareReorder(const nsBidiLevel *aLevels, PRInt32 aLength,
     aIndexMap[start]=start;
   }
 
-  return PR_TRUE;
+  return true;
 }
 
 #ifdef FULL_BIDI_ENGINE
 /* API functions for logical<->visual mapping ------------------------------- */
 
-nsresult nsBidi::GetVisualIndex(PRInt32 aLogicalIndex, PRInt32* aVisualIndex) {
+nsresult nsBidi::GetVisualIndex(int32_t aLogicalIndex, int32_t* aVisualIndex) {
   if(aLogicalIndex<0 || mLength<=aLogicalIndex) {
     return NS_ERROR_INVALID_ARG;
   } else {
@@ -1869,7 +1840,7 @@ nsresult nsBidi::GetVisualIndex(PRInt32 aLogicalIndex, PRInt32* aVisualIndex) {
           return NS_ERROR_OUT_OF_MEMORY;
         } else {
           Run *runs=mRuns;
-          PRInt32 i, visualStart=0, offset, length;
+          int32_t i, visualStart=0, offset, length;
 
           /* linear search for the run, search on the visual runs */
           for(i=0;; ++i) {
@@ -1893,7 +1864,7 @@ nsresult nsBidi::GetVisualIndex(PRInt32 aLogicalIndex, PRInt32* aVisualIndex) {
   }
 }
 
-nsresult nsBidi::GetLogicalIndex(PRInt32 aVisualIndex, PRInt32 *aLogicalIndex)
+nsresult nsBidi::GetLogicalIndex(int32_t aVisualIndex, int32_t *aLogicalIndex)
 {
   if(aVisualIndex<0 || mLength<=aVisualIndex) {
     return NS_ERROR_INVALID_ARG;
@@ -1911,14 +1882,14 @@ nsresult nsBidi::GetLogicalIndex(PRInt32 aVisualIndex, PRInt32 *aLogicalIndex)
           return NS_ERROR_OUT_OF_MEMORY;
         } else {
           Run *runs=mRuns;
-          PRInt32 i, runCount=mRunCount, start;
+          int32_t i, runCount=mRunCount, start;
 
           if(runCount<=10) {
             /* linear search for the run */
             for(i=0; aVisualIndex>=runs[i].visualLimit; ++i) {}
           } else {
             /* binary search for the run */
-            PRInt32 start=0, limit=runCount;
+            int32_t start=0, limit=runCount;
 
             /* the middle if() will guaranteed find the run, we don't need a loop limit */
             for(;;) {
@@ -1952,7 +1923,7 @@ nsresult nsBidi::GetLogicalIndex(PRInt32 aVisualIndex, PRInt32 *aLogicalIndex)
   }
 }
 
-nsresult nsBidi::GetLogicalMap(PRInt32 *aIndexMap)
+nsresult nsBidi::GetLogicalMap(int32_t *aIndexMap)
 {
   nsBidiLevel *levels;
   nsresult rv;
@@ -1961,28 +1932,28 @@ nsresult nsBidi::GetLogicalMap(PRInt32 *aIndexMap)
   rv = GetLevels(&levels);
   if(NS_FAILED(rv)) {
     return rv;
-  } else if(aIndexMap==NULL) {
+  } else if(aIndexMap==nullptr) {
     return NS_ERROR_INVALID_ARG;
   } else {
     return ReorderLogical(levels, mLength, aIndexMap);
   }
 }
 
-nsresult nsBidi::GetVisualMap(PRInt32 *aIndexMap)
+nsresult nsBidi::GetVisualMap(int32_t *aIndexMap)
 {
-  PRInt32* runCount=NULL;
+  int32_t* runCount=nullptr;
   nsresult rv;
 
   /* CountRuns() checks all of its and our arguments */
   rv = CountRuns(runCount);
   if(NS_FAILED(rv)) {
     return rv;
-  } else if(aIndexMap==NULL) {
+  } else if(aIndexMap==nullptr) {
     return NS_ERROR_INVALID_ARG;
   } else {
     /* fill a visual-to-logical index map using the runs[] */
     Run *runs=mRuns, *runsLimit=runs+mRunCount;
-    PRInt32 logicalStart, visualStart, visualLimit;
+    int32_t logicalStart, visualStart, visualLimit;
 
     visualStart=0;
     for(; runs<runsLimit; ++runs) {
@@ -2007,12 +1978,13 @@ nsresult nsBidi::GetVisualMap(PRInt32 *aIndexMap)
 
 /* reorder a line based on a levels array (L2) ------------------------------ */
 
-nsresult nsBidi::ReorderLogical(const nsBidiLevel *aLevels, PRInt32 aLength, PRInt32 *aIndexMap)
+nsresult nsBidi::ReorderLogical(const nsBidiLevel *aLevels, int32_t aLength, int32_t *aIndexMap)
 {
-  PRInt32 start, limit, sumOfSosEos;
+  int32_t start, limit, sumOfSosEos;
   nsBidiLevel minLevel, maxLevel;
 
-  if(aIndexMap==NULL || !PrepareReorder(aLevels, aLength, aIndexMap, &minLevel, &maxLevel)) {
+  if(aIndexMap==nullptr ||
+     !PrepareReorder(aLevels, aLength, aIndexMap, &minLevel, &maxLevel)) {
     return NS_OK;
   }
 
@@ -2072,9 +2044,9 @@ nsresult nsBidi::ReorderLogical(const nsBidiLevel *aLevels, PRInt32 aLength, PRI
   return NS_OK;
 }
 
-nsresult nsBidi::InvertMap(const PRInt32 *aSrcMap, PRInt32 *aDestMap, PRInt32 aLength)
+nsresult nsBidi::InvertMap(const int32_t *aSrcMap, int32_t *aDestMap, int32_t aLength)
 {
-  if(aSrcMap!=NULL && aDestMap!=NULL) {
+  if(aSrcMap!=nullptr && aDestMap!=nullptr) {
     aSrcMap+=aLength;
     while(aLength>0) {
       aDestMap[*--aSrcMap]=--aLength;
@@ -2083,8 +2055,8 @@ nsresult nsBidi::InvertMap(const PRInt32 *aSrcMap, PRInt32 *aDestMap, PRInt32 aL
   return NS_OK;
 }
 
-PRInt32 nsBidi::doWriteReverse(const PRUnichar *src, PRInt32 srcLength,
-                               PRUnichar *dest, PRUint16 options) {
+int32_t nsBidi::doWriteReverse(const char16_t *src, int32_t srcLength,
+                               char16_t *dest, uint16_t options) {
   /*
    * RTL run -
    *
@@ -2103,8 +2075,8 @@ PRInt32 nsBidi::doWriteReverse(const PRUnichar *src, PRInt32 srcLength,
    * whether characters should be replaced by their mirror-image
    * equivalent Unicode characters.
    */
-  PRInt32 i, j, destSize;
-  PRUint32 c;
+  int32_t i, j, destSize;
+  uint32_t c;
 
   /* optimize for several combinations of options */
   switch(options&(NSBIDI_REMOVE_BIDI_CONTROLS|NSBIDI_DO_MIRRORING|NSBIDI_KEEP_BASE_COMBINING)) {
@@ -2171,13 +2143,13 @@ PRInt32 nsBidi::doWriteReverse(const PRUnichar *src, PRInt32 srcLength,
       } else {
       /* we need to find out the destination length of the run,
                which will not include the Bidi control characters */
-        PRInt32 length=srcLength;
-        PRUnichar ch;
+        int32_t length=srcLength;
+        char16_t ch;
 
         i=0;
         do {
           ch=*src++;
-          if (!IsBidiControl((PRUint32)ch)) {
+          if (!IsBidiControl((uint32_t)ch)) {
             ++i;
           }
         } while(--length>0);
@@ -2210,7 +2182,7 @@ PRInt32 nsBidi::doWriteReverse(const PRUnichar *src, PRInt32 srcLength,
           /* mirror only the base character */
           c = SymmSwap(c);
 
-          PRInt32 k=0;
+          int32_t k=0;
           UTF_APPEND_CHAR_UNSAFE(dest, k, c);
           dest+=k;
           j+=k;
@@ -2224,10 +2196,10 @@ PRInt32 nsBidi::doWriteReverse(const PRUnichar *src, PRInt32 srcLength,
   return destSize;
 }
 
-nsresult nsBidi::WriteReverse(const PRUnichar *aSrc, PRInt32 aSrcLength, PRUnichar *aDest, PRUint16 aOptions, PRInt32 *aDestSize)
+nsresult nsBidi::WriteReverse(const char16_t *aSrc, int32_t aSrcLength, char16_t *aDest, uint16_t aOptions, int32_t *aDestSize)
 {
-  if( aSrc==NULL || aSrcLength<0 ||
-      aDest==NULL
+  if( aSrc==nullptr || aSrcLength<0 ||
+      aDest==nullptr
     ) {
     return NS_ERROR_INVALID_ARG;
   }
@@ -2245,4 +2217,3 @@ nsresult nsBidi::WriteReverse(const PRUnichar *aSrc, PRInt32 aSrcLength, PRUnich
   return NS_OK;
 }
 #endif // FULL_BIDI_ENGINE
-#endif // IBMBIDI
