@@ -1,53 +1,21 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /* struct containing the output from nsIFrame::Reflow */
 
 #ifndef nsHTMLReflowMetrics_h___
 #define nsHTMLReflowMetrics_h___
 
-#include <stdio.h>
-#include "nsISupports.h"
-#include "nsMargin.h"
 #include "nsRect.h"
 #include "nsBoundingMetrics.h"
+#include "WritingModes.h"
 
 //----------------------------------------------------------------------
 
+class nsHTMLReflowState;
+
 // Option flags
 #define NS_REFLOW_CALC_BOUNDING_METRICS  0x0001
 
@@ -71,11 +39,11 @@ private:
   nsRect mRects[2];
 public:
   nsRect& Overflow(size_t aIndex) {
-    NS_ASSERTION(0 <= aIndex && aIndex < 2, "index out of range");
+    NS_ASSERTION(aIndex < 2, "index out of range");
     return mRects[aIndex];
   }
   const nsRect& Overflow(size_t aIndex) const {
-    NS_ASSERTION(0 <= aIndex && aIndex < 2, "index out of range");
+    NS_ASSERTION(aIndex < 2, "index out of range");
     return mRects[aIndex];
   }
 
@@ -171,13 +139,13 @@ struct nsCollapsingMargin {
       {
       }
 
-    PRBool operator==(const nsCollapsingMargin& aOther)
+    bool operator==(const nsCollapsingMargin& aOther)
       {
         return mMostPos == aOther.mMostPos &&
           mMostNeg == aOther.mMostNeg;
       }
 
-    PRBool operator!=(const nsCollapsingMargin& aOther)
+    bool operator!=(const nsCollapsingMargin& aOther)
       {
         return !(*this == aOther);
       }
@@ -211,7 +179,7 @@ struct nsCollapsingMargin {
         mMostNeg = 0;
       }
 
-    PRBool IsZero() const
+    bool IsZero() const
       {
         return (mMostPos == 0) && (mMostNeg == 0);
       }
@@ -228,11 +196,87 @@ struct nsCollapsingMargin {
  *
  * @see #Reflow()
  */
-struct nsHTMLReflowMetrics {
-  nscoord width, height;    // [OUT] desired width and height (border-box)
-  nscoord ascent;           // [OUT] baseline (from top), or ASK_FOR_BASELINE
+class nsHTMLReflowMetrics {
+public:
+  // XXXldb Should |aFlags| generally be passed from parent to child?
+  // Some places do it, and some don't.  |aFlags| should perhaps go away
+  // entirely.
+  // XXX width/height/ascent are OUT parameters and so they shouldn't
+  // have to be initialized, but there are some bad frame classes that
+  // aren't properly setting them when returning from Reflow()...
+  nsHTMLReflowMetrics(mozilla::WritingMode aWritingMode, uint32_t aFlags = 0)
+    : mISize(0)
+    , mBSize(0)
+    , mBlockStartAscent(ASK_FOR_BASELINE)
+    , mFlags(aFlags)
+    , mWritingMode(aWritingMode)
+  {}
 
-  PRUint32 mFlags;
+  nsHTMLReflowMetrics(const nsHTMLReflowState& aState, uint32_t aFlags = 0);
+
+  // ISize and BSize are logical-coordinate dimensions:
+  // ISize is the size in the writing mode's inline direction (which equates to
+  // width in horizontal writing modes, height in vertical ones), and BSize is
+  // the size in the block-progression direction.
+  nscoord ISize() const { return mISize; }
+  nscoord BSize() const { return mBSize; }
+
+  nscoord& ISize() { return mISize; }
+  nscoord& BSize() { return mBSize; }
+
+  // Width and Height are physical dimensions, independent of writing mode.
+  // Accessing these is slightly more expensive than accessing the logical
+  // dimensions (once vertical writing mode support is enabled); as far as
+  // possible, client code should work purely with logical dimensions.
+  nscoord Width() const { return mWritingMode.IsVertical() ? mBSize : mISize; }
+  nscoord Height() const { return mWritingMode.IsVertical() ? mISize : mBSize; }
+
+  // It's only meaningful to consider "ascent" on the block-start side of the
+  // frame; asking for the "ascent" on any other side will just return zero.
+  nscoord TopAscent() const
+  {
+    return mWritingMode.IsVertical() ? 0 : mBlockStartAscent;
+  }
+  nscoord LeftAscent() const
+  {
+    return mWritingMode.IsVertical() && mWritingMode.IsVerticalLR() ?
+           mBlockStartAscent : 0;
+  }
+  nscoord RightAscent() const
+  {
+    return mWritingMode.IsVertical() && !mWritingMode.IsVerticalLR() ?
+           mBlockStartAscent : 0;
+  }
+
+  nscoord& Width() { return mWritingMode.IsVertical() ? mBSize : mISize; }
+  nscoord& Height() { return mWritingMode.IsVertical() ? mISize : mBSize; }
+
+  // To set the ascent value, we must be sure we're working with the correct
+  // writing mode, so either pass it to the logical setter...
+  void SetBlockStartAscent(mozilla::WritingMode aWritingMode, nscoord aAscent)
+  {
+    NS_ASSERTION(aWritingMode == mWritingMode, "writing mode mismatch");
+    mBlockStartAscent = aAscent;
+  }
+  // ...or call the appropriate physical setter (these will probably be removed
+  // eventually).
+  void SetTopAscent(nscoord aAscent)
+  {
+    NS_ASSERTION(!mWritingMode.IsVertical(), "writing mode mismatch");
+    mBlockStartAscent = aAscent;
+  }
+  void SetLeftAscent(nscoord aAscent)
+  {
+    NS_ASSERTION(mWritingMode.IsVertical() && mWritingMode.IsVerticalLR(),
+                 "writing mode mismatch");
+    mBlockStartAscent = aAscent;
+  }
+  void SetRightAscent(nscoord aAscent)
+  {
+    NS_ASSERTION(mWritingMode.IsVertical() && !mWritingMode.IsVerticalLR(),
+                 "writing mode mismatch");
+    mBlockStartAscent = aAscent;
+  }
 
   enum { ASK_FOR_BASELINE = nscoord_MAX };
 
@@ -273,15 +317,17 @@ struct nsHTMLReflowMetrics {
   // Union all of mOverflowAreas with (0, 0, width, height).
   void UnionOverflowAreasWithDesiredBounds();
 
-  // XXXldb Should |aFlags| generally be passed from parent to child?
-  // Some places do it, and some don't.  |aFlags| should perhaps go away
-  // entirely.
-  // XXX width/height/ascent are OUT parameters and so they shouldn't
-  // have to be initialized, but there are some bad frame classes that
-  // aren't properly setting them when returning from Reflow()...
-  nsHTMLReflowMetrics(PRUint32 aFlags = 0)
-    : width(0), height(0), ascent(ASK_FOR_BASELINE), mFlags(aFlags)
-  {}
+  mozilla::WritingMode GetWritingMode() const { return mWritingMode; }
+
+private:
+  nscoord mISize, mBSize; // [OUT] desired width and height (border-box)
+  nscoord mBlockStartAscent; // [OUT] baseline (in Block direction), or ASK_FOR_BASELINE
+
+public:
+  uint32_t mFlags;
+
+private:
+  mozilla::WritingMode mWritingMode;
 };
 
 #endif /* nsHTMLReflowMetrics_h___ */
