@@ -1,53 +1,27 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /* DOM object representing values in DOM computed style */
 
 #include "nsROCSSPrimitiveValue.h"
 
+#include "mozilla/dom/CSSPrimitiveValueBinding.h"
 #include "nsPresContext.h"
 #include "nsStyleUtil.h"
 #include "nsDOMCSSRGBColor.h"
-#include "nsIDOMRect.h"
+#include "nsDOMCSSRect.h"
+#include "nsIURI.h"
+#include "nsError.h"
+
+using namespace mozilla;
 
 nsROCSSPrimitiveValue::nsROCSSPrimitiveValue()
-  : mType(CSS_PX)
+  : CSSValue(), mType(CSS_PX)
 {
   mValue.mAppUnits = 0;
+  SetIsDOMBinding();
 }
 
 
@@ -56,20 +30,43 @@ nsROCSSPrimitiveValue::~nsROCSSPrimitiveValue()
   Reset();
 }
 
-NS_IMPL_ADDREF(nsROCSSPrimitiveValue)
-NS_IMPL_RELEASE(nsROCSSPrimitiveValue)
+NS_IMPL_CYCLE_COLLECTING_ADDREF(nsROCSSPrimitiveValue)
+NS_IMPL_CYCLE_COLLECTING_RELEASE(nsROCSSPrimitiveValue)
 
 
-DOMCI_DATA(ROCSSPrimitiveValue, nsROCSSPrimitiveValue)
-
 // QueryInterface implementation for nsROCSSPrimitiveValue
-NS_INTERFACE_MAP_BEGIN(nsROCSSPrimitiveValue)
+NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(nsROCSSPrimitiveValue)
+  NS_WRAPPERCACHE_INTERFACE_MAP_ENTRY
   NS_INTERFACE_MAP_ENTRY(nsIDOMCSSPrimitiveValue)
   NS_INTERFACE_MAP_ENTRY(nsIDOMCSSValue)
-  NS_INTERFACE_MAP_ENTRY(nsISupports)
-  NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO(ROCSSPrimitiveValue)
+  NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, CSSValue)
 NS_INTERFACE_MAP_END
 
+NS_IMPL_CYCLE_COLLECTION_CLASS(nsROCSSPrimitiveValue)
+
+NS_IMPL_CYCLE_COLLECTION_TRACE_WRAPPERCACHE(nsROCSSPrimitiveValue)
+
+NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(nsROCSSPrimitiveValue)
+  if (tmp->mType == CSS_URI) {
+    NS_IMPL_CYCLE_COLLECTION_TRAVERSE_RAWPTR(mValue.mURI)
+  } else if (tmp->mType == CSS_RGBCOLOR) {
+    NS_IMPL_CYCLE_COLLECTION_TRAVERSE_RAWPTR(mValue.mColor)
+  } else if (tmp->mType == CSS_RECT) {
+    NS_IMPL_CYCLE_COLLECTION_TRAVERSE_RAWPTR(mValue.mRect)
+  }
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_SCRIPT_OBJECTS
+NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
+
+NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(nsROCSSPrimitiveValue)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK_PRESERVED_WRAPPER
+  tmp->Reset();
+NS_IMPL_CYCLE_COLLECTION_UNLINK_END
+
+JSObject*
+nsROCSSPrimitiveValue::WrapObject(JSContext *cx)
+{
+  return dom::CSSPrimitiveValueBinding::Wrap(cx, this);
+}
 
 // nsIDOMCSSValue
 
@@ -85,7 +82,7 @@ nsROCSSPrimitiveValue::GetCssText(nsAString& aCssText)
     case CSS_PX :
       {
         float val = nsPresContext::AppUnitsToFloatCSSPixels(mValue.mAppUnits);
-        tmpStr.AppendFloat(val);
+        nsStyleUtil::AppendCSSNumber(val, tmpStr);
         tmpStr.AppendLiteral("px");
         break;
       }
@@ -104,7 +101,7 @@ nsROCSSPrimitiveValue::GetCssText(nsAString& aCssText)
     case CSS_URI :
       {
         if (mValue.mURI) {
-          nsCAutoString specUTF8;
+          nsAutoCString specUTF8;
           mValue.mURI->GetSpec(specUTF8);
 
           tmpStr.AssignLiteral("url(");
@@ -112,9 +109,11 @@ nsROCSSPrimitiveValue::GetCssText(nsAString& aCssText)
                                               tmpStr);
           tmpStr.AppendLiteral(")");
         } else {
-          // XXXldb Any better ideas?  It's good to have something that
-          // doesn't parse so that things round-trip "correctly".
-          tmpStr.Assign(NS_LITERAL_STRING("url(invalid-url:)"));
+          // http://dev.w3.org/csswg/css3-values/#attr defines
+          // 'about:invalid' as the default value for url attributes,
+          // so let's also use it here as the default computed value
+          // for invalid URLs.
+          tmpStr.Assign(NS_LITERAL_STRING("url(about:invalid)"));
         }
         break;
       }
@@ -122,18 +121,52 @@ nsROCSSPrimitiveValue::GetCssText(nsAString& aCssText)
       {
         tmpStr.AppendLiteral("attr(");
         tmpStr.Append(mValue.mString);
-        tmpStr.Append(PRUnichar(')'));
+        tmpStr.Append(char16_t(')'));
         break;
       }
     case CSS_PERCENTAGE :
       {
-        tmpStr.AppendFloat(mValue.mFloat * 100);
-        tmpStr.Append(PRUnichar('%'));
+        nsStyleUtil::AppendCSSNumber(mValue.mFloat * 100, tmpStr);
+        tmpStr.Append(char16_t('%'));
         break;
       }
     case CSS_NUMBER :
       {
-        tmpStr.AppendFloat(mValue.mFloat);
+        nsStyleUtil::AppendCSSNumber(mValue.mFloat, tmpStr);
+        break;
+      }
+    case CSS_NUMBER_INT32 :
+      {
+        tmpStr.AppendInt(mValue.mInt32);
+        break;
+      }
+    case CSS_NUMBER_UINT32 :
+      {
+        tmpStr.AppendInt(mValue.mUint32);
+        break;
+      }
+    case CSS_DEG :
+      {
+        nsStyleUtil::AppendCSSNumber(mValue.mFloat, tmpStr);
+        tmpStr.AppendLiteral("deg");
+        break;
+      }
+    case CSS_GRAD :
+      {
+        nsStyleUtil::AppendCSSNumber(mValue.mFloat, tmpStr);
+        tmpStr.AppendLiteral("grad");
+        break;
+      }
+    case CSS_RAD :
+      {
+        nsStyleUtil::AppendCSSNumber(mValue.mFloat, tmpStr);
+        tmpStr.AppendLiteral("rad");
+        break;
+      }
+    case CSS_TURN :
+      {
+        nsStyleUtil::AppendCSSNumber(mValue.mFloat, tmpStr);
+        tmpStr.AppendLiteral("turn");
         break;
       }
     case CSS_RECT :
@@ -180,8 +213,8 @@ nsROCSSPrimitiveValue::GetCssText(nsAString& aCssText)
     case CSS_RGBCOLOR :
       {
         NS_ASSERTION(mValue.mColor, "mValue.mColor should never be null");
+        ErrorResult error;
         NS_NAMED_LITERAL_STRING(comma, ", ");
-        nsCOMPtr<nsIDOMCSSPrimitiveValue> colorCSSValue;
         nsAutoString colorValue;
         if (mValue.mColor->HasAlpha())
           tmpStr.AssignLiteral("rgba(");
@@ -189,39 +222,27 @@ nsROCSSPrimitiveValue::GetCssText(nsAString& aCssText)
           tmpStr.AssignLiteral("rgb(");
 
         // get the red component
-        result = mValue.mColor->GetRed(getter_AddRefs(colorCSSValue));
-        if (NS_FAILED(result))
-          break;
-        result = colorCSSValue->GetCssText(colorValue);
-        if (NS_FAILED(result))
+        mValue.mColor->Red()->GetCssText(colorValue, error);
+        if (error.Failed())
           break;
         tmpStr.Append(colorValue + comma);
 
         // get the green component
-        result = mValue.mColor->GetGreen(getter_AddRefs(colorCSSValue));
-        if (NS_FAILED(result))
-          break;
-        result = colorCSSValue->GetCssText(colorValue);
-        if (NS_FAILED(result))
+        mValue.mColor->Green()->GetCssText(colorValue, error);
+        if (error.Failed())
           break;
         tmpStr.Append(colorValue + comma);
 
         // get the blue component
-        result = mValue.mColor->GetBlue(getter_AddRefs(colorCSSValue));
-        if (NS_FAILED(result))
-          break;
-        result = colorCSSValue->GetCssText(colorValue);
-        if (NS_FAILED(result))
+        mValue.mColor->Blue()->GetCssText(colorValue, error);
+        if (error.Failed())
           break;
         tmpStr.Append(colorValue);
 
         if (mValue.mColor->HasAlpha()) {
           // get the alpha component
-          result = mValue.mColor->GetAlpha(getter_AddRefs(colorCSSValue));
-          if (NS_FAILED(result))
-            break;
-          result = colorCSSValue->GetCssText(colorValue);
-          if (NS_FAILED(result))
+          mValue.mColor->Alpha()->GetCssText(colorValue, error);
+          if (error.Failed())
             break;
           tmpStr.Append(comma + colorValue);
         }
@@ -232,7 +253,7 @@ nsROCSSPrimitiveValue::GetCssText(nsAString& aCssText)
       }
     case CSS_S :
       {
-        tmpStr.AppendFloat(mValue.mFloat);
+        nsStyleUtil::AppendCSSNumber(mValue.mFloat, tmpStr);
         tmpStr.AppendLiteral("s");
         break;
       }
@@ -244,9 +265,6 @@ nsROCSSPrimitiveValue::GetCssText(nsAString& aCssText)
     case CSS_UNKNOWN :
     case CSS_EMS :
     case CSS_EXS :
-    case CSS_DEG :
-    case CSS_RAD :
-    case CSS_GRAD :
     case CSS_MS :
     case CSS_HZ :
     case CSS_KHZ :
@@ -262,6 +280,11 @@ nsROCSSPrimitiveValue::GetCssText(nsAString& aCssText)
   return NS_OK;
 }
 
+void
+nsROCSSPrimitiveValue::GetCssText(nsString& aText, ErrorResult& aRv)
+{
+  aRv = GetCssText(aText);
+}
 
 NS_IMETHODIMP
 nsROCSSPrimitiveValue::SetCssText(const nsAString& aCssText)
@@ -269,85 +292,114 @@ nsROCSSPrimitiveValue::SetCssText(const nsAString& aCssText)
   return NS_ERROR_DOM_NO_MODIFICATION_ALLOWED_ERR;
 }
 
+void
+nsROCSSPrimitiveValue::SetCssText(const nsAString& aText, ErrorResult& aRv)
+{
+  aRv = SetCssText(aText);
+}
+
 
 NS_IMETHODIMP
-nsROCSSPrimitiveValue::GetCssValueType(PRUint16* aValueType)
+nsROCSSPrimitiveValue::GetCssValueType(uint16_t* aValueType)
 {
   NS_ENSURE_ARG_POINTER(aValueType);
   *aValueType = nsIDOMCSSValue::CSS_PRIMITIVE_VALUE;
   return NS_OK;
 }
 
+uint16_t
+nsROCSSPrimitiveValue::CssValueType() const
+{
+  return nsIDOMCSSValue::CSS_PRIMITIVE_VALUE;
+}
+
 
 // nsIDOMCSSPrimitiveValue
 
 NS_IMETHODIMP
-nsROCSSPrimitiveValue::GetPrimitiveType(PRUint16* aPrimitiveType)
+nsROCSSPrimitiveValue::GetPrimitiveType(uint16_t* aPrimitiveType)
 {
   NS_ENSURE_ARG_POINTER(aPrimitiveType);
-  *aPrimitiveType = mType;
+  *aPrimitiveType = PrimitiveType();
 
   return NS_OK;
 }
 
 
 NS_IMETHODIMP
-nsROCSSPrimitiveValue::SetFloatValue(PRUint16 aUnitType, float aFloatValue)
+nsROCSSPrimitiveValue::SetFloatValue(uint16_t aUnitType, float aFloatValue)
 {
   return NS_ERROR_DOM_NO_MODIFICATION_ALLOWED_ERR;
 }
 
-
-NS_IMETHODIMP
-nsROCSSPrimitiveValue::GetFloatValue(PRUint16 aUnitType, float* aReturn)
+void
+nsROCSSPrimitiveValue::SetFloatValue(uint16_t aType, float aVal,
+                                     ErrorResult& aRv)
 {
-  NS_ENSURE_ARG_POINTER(aReturn);
-  *aReturn = 0;
+  aRv = SetFloatValue(aType, aVal);
+}
 
+float
+nsROCSSPrimitiveValue::GetFloatValue(uint16_t aUnitType, ErrorResult& aRv)
+{
   switch(aUnitType) {
     case CSS_PX :
-      if (mType != CSS_PX)
-        return NS_ERROR_DOM_INVALID_ACCESS_ERR;
-      *aReturn = nsPresContext::AppUnitsToFloatCSSPixels(mValue.mAppUnits);
+      if (mType == CSS_PX) {
+        return nsPresContext::AppUnitsToFloatCSSPixels(mValue.mAppUnits);
+      }
+
       break;
     case CSS_CM :
-      if (mType != CSS_PX)
-        return NS_ERROR_DOM_INVALID_ACCESS_ERR;
-      *aReturn = mValue.mAppUnits * CM_PER_INCH_FLOAT /
-        nsPresContext::AppUnitsPerCSSInch();
+      if (mType == CSS_PX) {
+        return mValue.mAppUnits * CM_PER_INCH_FLOAT /
+          nsPresContext::AppUnitsPerCSSInch();
+      }
+
       break;
     case CSS_MM :
-      if (mType != CSS_PX)
-        return NS_ERROR_DOM_INVALID_ACCESS_ERR;
-      *aReturn = mValue.mAppUnits * MM_PER_INCH_FLOAT /
-        nsPresContext::AppUnitsPerCSSInch();
+      if (mType == CSS_PX) {
+        return mValue.mAppUnits * MM_PER_INCH_FLOAT /
+          nsPresContext::AppUnitsPerCSSInch();
+      }
+
       break;
     case CSS_IN :
-      if (mType != CSS_PX)
-        return NS_ERROR_DOM_INVALID_ACCESS_ERR;
-      *aReturn = mValue.mAppUnits / nsPresContext::AppUnitsPerCSSInch();
+      if (mType == CSS_PX) {
+        return mValue.mAppUnits / nsPresContext::AppUnitsPerCSSInch();
+      }
+
       break;
     case CSS_PT :
-      if (mType != CSS_PX)
-        return NS_ERROR_DOM_INVALID_ACCESS_ERR;
-      *aReturn = mValue.mAppUnits * POINTS_PER_INCH_FLOAT / 
-        nsPresContext::AppUnitsPerCSSInch();
+      if (mType == CSS_PX) {
+        return mValue.mAppUnits * POINTS_PER_INCH_FLOAT /
+          nsPresContext::AppUnitsPerCSSInch();
+      }
+
       break;
     case CSS_PC :
-      if (mType != CSS_PX)
-        return NS_ERROR_DOM_INVALID_ACCESS_ERR;
-      *aReturn = mValue.mAppUnits * 6.0f /
-        nsPresContext::AppUnitsPerCSSInch();
+      if (mType == CSS_PX) {
+        return mValue.mAppUnits * 6.0f /
+          nsPresContext::AppUnitsPerCSSInch();
+      }
+
       break;
     case CSS_PERCENTAGE :
-      if (mType != CSS_PERCENTAGE)
-        return NS_ERROR_DOM_INVALID_ACCESS_ERR;
-      *aReturn = mValue.mFloat * 100;
+      if (mType == CSS_PERCENTAGE) {
+        return mValue.mFloat * 100;
+      }
+
       break;
     case CSS_NUMBER :
-      if (mType != CSS_NUMBER)
-        return NS_ERROR_DOM_INVALID_ACCESS_ERR;
-      *aReturn = mValue.mFloat;
+      if (mType == CSS_NUMBER) {
+        return mValue.mFloat;
+      }
+      if (mType == CSS_NUMBER_INT32) {
+        return mValue.mInt32;
+      }
+      if (mType == CSS_NUMBER_UINT32) {
+        return mValue.mUint32;
+      }
+
       break;
     case CSS_UNKNOWN :
     case CSS_EMS :
@@ -367,20 +419,36 @@ nsROCSSPrimitiveValue::GetFloatValue(PRUint16 aUnitType, float* aReturn)
     case CSS_COUNTER :
     case CSS_RECT :
     case CSS_RGBCOLOR :
-      return NS_ERROR_DOM_INVALID_ACCESS_ERR;
+      break;
   }
 
-  return NS_OK;
+  aRv.Throw(NS_ERROR_DOM_INVALID_ACCESS_ERR);
+  return 0;
+}
+
+NS_IMETHODIMP
+nsROCSSPrimitiveValue::GetFloatValue(uint16_t aType, float *aVal)
+{
+  ErrorResult rv;
+  *aVal = GetFloatValue(aType, rv);
+  return rv.ErrorCode();
 }
 
 
 NS_IMETHODIMP
-nsROCSSPrimitiveValue::SetStringValue(PRUint16 aStringType,
+nsROCSSPrimitiveValue::SetStringValue(uint16_t aStringType,
                                       const nsAString& aStringValue)
 {
   return NS_ERROR_DOM_NO_MODIFICATION_ALLOWED_ERR;
 }
 
+void
+nsROCSSPrimitiveValue::SetStringValue(uint16_t aType, const nsAString& aString,
+                                      mozilla::ErrorResult& aRv)
+{
+  aRv.Throw(NS_ERROR_DOM_NO_MODIFICATION_ALLOWED_ERR);
+}
+
 
 NS_IMETHODIMP
 nsROCSSPrimitiveValue::GetStringValue(nsAString& aReturn)
@@ -394,7 +462,7 @@ nsROCSSPrimitiveValue::GetStringValue(nsAString& aReturn)
       aReturn.Assign(mValue.mString);
       break;
     case CSS_URI: {
-      nsCAutoString spec;
+      nsAutoCString spec;
       if (mValue.mURI)
         mValue.mURI->GetSpec(spec);
       CopyUTF8toUTF16(spec, aReturn);
@@ -406,6 +474,12 @@ nsROCSSPrimitiveValue::GetStringValue(nsAString& aReturn)
   return NS_OK;
 }
 
+void
+nsROCSSPrimitiveValue::GetStringValue(nsString& aString, ErrorResult& aRv)
+{
+  aRv = GetStringValue(aString);
+}
+
 
 NS_IMETHODIMP
 nsROCSSPrimitiveValue::GetCounterValue(nsIDOMCounter** aReturn)
@@ -413,30 +487,43 @@ nsROCSSPrimitiveValue::GetCounterValue(nsIDOMCounter** aReturn)
   return NS_ERROR_DOM_NO_MODIFICATION_ALLOWED_ERR;
 }
 
-
-NS_IMETHODIMP
-nsROCSSPrimitiveValue::GetRectValue(nsIDOMRect** aReturn)
+already_AddRefed<nsIDOMCounter>
+nsROCSSPrimitiveValue::GetCounterValue(ErrorResult& aRv)
 {
-  if (mType != CSS_RECT) {
-    *aReturn = nsnull;
-    return NS_ERROR_DOM_INVALID_ACCESS_ERR;
-  }
-  NS_ASSERTION(mValue.mRect, "mValue.mRect should never be null");
-  NS_ADDREF(*aReturn = mValue.mRect);
-  return NS_OK;
+  aRv.Throw(NS_ERROR_DOM_NO_MODIFICATION_ALLOWED_ERR);
+  return nullptr;
 }
 
+nsDOMCSSRect*
+nsROCSSPrimitiveValue::GetRectValue(ErrorResult& aRv)
+{
+  if (mType != CSS_RECT) {
+    aRv.Throw(NS_ERROR_DOM_INVALID_ACCESS_ERR);
+    return nullptr;
+  }
 
-NS_IMETHODIMP 
-nsROCSSPrimitiveValue::GetRGBColorValue(nsIDOMRGBColor** aReturn)
+  NS_ASSERTION(mValue.mRect, "mValue.mRect should never be null");
+  return mValue.mRect;
+}
+
+NS_IMETHODIMP
+nsROCSSPrimitiveValue::GetRectValue(nsIDOMRect** aRect)
+{
+  ErrorResult error;
+  NS_IF_ADDREF(*aRect = GetRectValue(error));
+  return error.ErrorCode();
+}
+
+nsDOMCSSRGBColor*
+nsROCSSPrimitiveValue::GetRGBColorValue(ErrorResult& aRv)
 {
   if (mType != CSS_RGBCOLOR) {
-    *aReturn = nsnull;
-    return NS_ERROR_DOM_INVALID_ACCESS_ERR;
+    aRv.Throw(NS_ERROR_DOM_INVALID_ACCESS_ERR);
+    return nullptr;
   }
+
   NS_ASSERTION(mValue.mColor, "mValue.mColor should never be null");
-  NS_ADDREF(*aReturn = mValue.mColor);
-  return NS_OK;
+  return mValue.mColor;
 }
 
 void
@@ -448,19 +535,19 @@ nsROCSSPrimitiveValue::SetNumber(float aValue)
 }
 
 void
-nsROCSSPrimitiveValue::SetNumber(PRInt32 aValue)
+nsROCSSPrimitiveValue::SetNumber(int32_t aValue)
 {
   Reset();
-  mValue.mFloat = float(aValue);
-  mType = CSS_NUMBER;
+  mValue.mInt32 = aValue;
+  mType = CSS_NUMBER_INT32;
 }
 
 void
-nsROCSSPrimitiveValue::SetNumber(PRUint32 aValue)
+nsROCSSPrimitiveValue::SetNumber(uint32_t aValue)
 {
   Reset();
-  mValue.mFloat = float(aValue);
-  mType = CSS_NUMBER;
+  mValue.mUint32 = aValue;
+  mType = CSS_NUMBER_UINT32;
 }
 
 void
@@ -471,6 +558,38 @@ nsROCSSPrimitiveValue::SetPercent(float aValue)
   mType = CSS_PERCENTAGE;
 }
 
+void
+nsROCSSPrimitiveValue::SetDegree(float aValue)
+{
+  Reset();
+  mValue.mFloat = aValue;
+  mType = CSS_DEG;
+}
+
+void
+nsROCSSPrimitiveValue::SetGrad(float aValue)
+{
+  Reset();
+  mValue.mFloat = aValue;
+  mType = CSS_GRAD;
+}
+
+void
+nsROCSSPrimitiveValue::SetRadian(float aValue)
+{
+  Reset();
+  mValue.mFloat = aValue;
+  mType = CSS_RAD;
+}
+
+void
+nsROCSSPrimitiveValue::SetTurn(float aValue)
+{
+  Reset();
+  mValue.mFloat = aValue;
+  mType = CSS_TURN;
+}
+
 void
 nsROCSSPrimitiveValue::SetAppUnits(nscoord aValue)
 {
@@ -498,7 +617,7 @@ nsROCSSPrimitiveValue::SetIdent(nsCSSKeyword aKeyword)
 
 // FIXME: CSS_STRING should imply a string with "" and a need for escaping.
 void
-nsROCSSPrimitiveValue::SetString(const nsACString& aString, PRUint16 aType)
+nsROCSSPrimitiveValue::SetString(const nsACString& aString, uint16_t aType)
 {
   Reset();
   mValue.mString = ToNewUnicode(aString);
@@ -512,7 +631,7 @@ nsROCSSPrimitiveValue::SetString(const nsACString& aString, PRUint16 aType)
 
 // FIXME: CSS_STRING should imply a string with "" and a need for escaping.
 void
-nsROCSSPrimitiveValue::SetString(const nsAString& aString, PRUint16 aType)
+nsROCSSPrimitiveValue::SetString(const nsAString& aString, uint16_t aType)
 {
   Reset();
   mValue.mString = ToNewUnicode(aString);
@@ -549,7 +668,7 @@ nsROCSSPrimitiveValue::SetColor(nsDOMCSSRGBColor* aColor)
 }
 
 void
-nsROCSSPrimitiveValue::SetRect(nsIDOMRect* aRect)
+nsROCSSPrimitiveValue::SetRect(nsDOMCSSRect* aRect)
 {
   NS_PRECONDITION(aRect, "Null rect being set!");
   Reset();
@@ -582,7 +701,7 @@ nsROCSSPrimitiveValue::Reset()
     case CSS_COUNTER: // FIXME: Counter should use an object
       NS_ASSERTION(mValue.mString, "Null string should never happen");
       nsMemory::Free(mValue.mString);
-      mValue.mString = nsnull;
+      mValue.mString = nullptr;
       break;
     case CSS_URI:
       NS_IF_RELEASE(mValue.mURI);
@@ -596,4 +715,6 @@ nsROCSSPrimitiveValue::Reset()
       NS_RELEASE(mValue.mColor);
       break;
   }
+
+  mType = CSS_UNKNOWN;
 }
