@@ -1,46 +1,7 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Communicator client code, released
- * March 31, 1998.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Daniel Veditz <dveditz@netscape.com>
- *   Samir Gehani <sgehani@netscape.com>
- *   Mitch Stoltz <mstoltz@netscape.com>
- *   Jeroen Dobbelaere <jeroen.dobbelaere@acunia.com>
- *   Jeff Walden <jwalden+code@mit.edu>
- *   Taras Glek <tglek@mozilla.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /*
  * This module implements a simple archive extractor for the PKZIP format.
@@ -49,10 +10,14 @@
  * or pointers to it across thread boundaries.
  */
 
-#define READTYPE  PRInt32
+// This must be the first include in the file in order for the
+// PL_ARENA_CONST_ALIGN_MASK macro to be effective.
+#define PL_ARENA_CONST_ALIGN_MASK  (sizeof(void*)-1)
+#include "plarena.h"
+
+#define READTYPE  int32_t
 #include "zlib.h"
 #include "nsISupportsUtils.h"
-#include "nsRecyclingAllocator.h"
 #include "prio.h"
 #include "plstr.h"
 #include "prlog.h"
@@ -60,29 +25,22 @@
 #include "nsWildCard.h"
 #include "nsZipArchive.h"
 #include "nsString.h"
-#include "mozilla/FunctionTimer.h"
 #include "prenv.h"
 #if defined(XP_WIN)
 #include <windows.h>
 #endif
-#include "mozilla/Telemetry.h"
-
-/**
- * Global allocator used with zlib. Destroyed in module shutdown.
- */
-#define NBUCKETS 6
-nsRecyclingAllocator *gZlibAllocator = NULL;
 
 // For placement new used for arena allocations of zip file list
-#include NEW_H
+#include <new>
 #define ZIP_ARENABLOCKSIZE (1*1024)
 
 #ifdef XP_UNIX
+    #include <sys/mman.h>
     #include <sys/types.h>
     #include <sys/stat.h>
     #include <limits.h>
     #include <unistd.h>
-#elif defined(XP_WIN) || defined(XP_OS2)
+#elif defined(XP_WIN)
     #include <io.h>
 #endif
 
@@ -103,62 +61,98 @@ nsRecyclingAllocator *gZlibAllocator = NULL;
 #  endif
 #endif  /* XP_UNIX */
 
+#ifdef XP_WIN
+#include "private/pprio.h"  // To get PR_ImportFile
+#endif
 
 using namespace mozilla;
 
-static const PRUint32 kMaxNameLength = PATH_MAX; /* Maximum name length */
+static const uint32_t kMaxNameLength = PATH_MAX; /* Maximum name length */
 // For synthetic zip entries. Date/time corresponds to 1980-01-01 00:00.
-static const PRUint16 kSyntheticTime = 0;
-static const PRUint16 kSyntheticDate = (1 + (1 << 5) + (0 << 9));
+static const uint16_t kSyntheticTime = 0;
+static const uint16_t kSyntheticDate = (1 + (1 << 5) + (0 << 9));
 
-static PRUint16 xtoint(const PRUint8 *ii);
-static PRUint32 xtolong(const PRUint8 *ll);
-static PRUint32 HashName(const char* aName, PRUint16 nameLen);
+static uint16_t xtoint(const uint8_t *ii);
+static uint32_t xtolong(const uint8_t *ll);
+static uint32_t HashName(const char* aName, uint16_t nameLen);
 #ifdef XP_UNIX
 static nsresult ResolveSymlink(const char *path);
 #endif
 
-//***********************************************************
-// Allocators for use with zlib
-//
-// Use a recycling allocator, for re-use of of the zlib buffers.
-// For every inflation the following allocations are done:
-// zlibAlloc(1, 9520)
-// zlibAlloc(32768, 1)
-//***********************************************************
+class ZipArchiveLogger {
+public:
+  void Write(const nsACString &zip, const char *entry) const {
+    if (!fd) {
+      char *env = PR_GetEnv("MOZ_JAR_LOG_FILE");
+      if (!env)
+        return;
 
-static void *
-zlibAlloc(void *opaque, uInt items, uInt size)
-{
-  nsRecyclingAllocator *zallocator = (nsRecyclingAllocator *)opaque;
-  if (zallocator) {
-    return gZlibAllocator->Malloc(items * size);
+      nsCOMPtr<nsIFile> logFile;
+      nsresult rv = NS_NewLocalFile(NS_ConvertUTF8toUTF16(env), false, getter_AddRefs(logFile));
+      if (NS_FAILED(rv))
+        return;
+
+      // Create the log file and its parent directory (in case it doesn't exist)
+      logFile->Create(nsIFile::NORMAL_FILE_TYPE, 0644);
+
+      PRFileDesc* file;
+#ifdef XP_WIN
+      // PR_APPEND is racy on Windows, so open a handle ourselves with flags that
+      // will work, and use PR_ImportFile to make it a PRFileDesc.
+      // This can go away when bug 840435 is fixed.
+      nsAutoString path;
+      logFile->GetPath(path);
+      if (path.IsEmpty())
+        return;
+      HANDLE handle = CreateFileW(path.get(), FILE_APPEND_DATA, FILE_SHARE_WRITE,
+                                  nullptr, OPEN_ALWAYS, 0, nullptr);
+      if (handle == INVALID_HANDLE_VALUE)
+        return;
+      file = PR_ImportFile((PROsfd)handle);
+      if (!file)
+        return;
+#else
+      rv = logFile->OpenNSPRFileDesc(PR_WRONLY|PR_CREATE_FILE|PR_APPEND, 0644, &file);
+      if (NS_FAILED(rv))
+        return;
+#endif
+      fd = file;
+    }
+    nsCString buf(zip);
+    buf.Append(" ");
+    buf.Append(entry);
+    buf.Append('\n');
+    PR_Write(fd, buf.get(), buf.Length());
   }
-  return malloc(items * size);
-}
 
-static void
-zlibFree(void *opaque, void *ptr)
-{
-  nsRecyclingAllocator *zallocator = (nsRecyclingAllocator *)opaque;
-  if (zallocator)
-    zallocator->Free(ptr);
-  else
-    free(ptr);
-}
+  void AddRef() {
+    MOZ_ASSERT(refCnt >= 0);
+    ++refCnt;
+  }
+
+  void Release() {
+    MOZ_ASSERT(refCnt > 0);
+    if ((0 == --refCnt) && fd) {
+      PR_Close(fd);
+      fd = nullptr;
+    }
+  }
+private:
+  int refCnt;
+  mutable PRFileDesc *fd;
+};
+
+static ZipArchiveLogger zipLog;
+
+//***********************************************************
+// For every inflation the following allocations are done:
+// malloc(1 * 9520)
+// malloc(32768 * 1)
+//***********************************************************
 
 nsresult gZlibInit(z_stream *zs)
 {
   memset(zs, 0, sizeof(z_stream));
-  //-- ensure we have our zlib allocator for better performance
-  if (!gZlibAllocator) {
-    gZlibAllocator = new nsRecyclingAllocator(NBUCKETS, NS_DEFAULT_RECYCLE_TIMEOUT, "libjar");
-  }
-  if (gZlibAllocator) {
-    zs->zalloc = zlibAlloc;
-    zs->zfree = zlibFree;
-    zs->opaque = gZlibAllocator;
-  }
   int zerr = inflateInit2(zs, -MAX_WBITS);
   if (zerr != Z_OK) return NS_ERROR_OUT_OF_MEMORY;
 
@@ -166,33 +160,37 @@ nsresult gZlibInit(z_stream *zs)
 }
 
 nsZipHandle::nsZipHandle()
-  : mFileData(nsnull)
+  : mFileData(nullptr)
   , mLen(0)
-  , mMap(nsnull)
+  , mMap(nullptr)
   , mRefCnt(0)
 {
   MOZ_COUNT_CTOR(nsZipHandle);
 }
 
-NS_IMPL_THREADSAFE_ADDREF(nsZipHandle)
-NS_IMPL_THREADSAFE_RELEASE(nsZipHandle)
+NS_IMPL_ADDREF(nsZipHandle)
+NS_IMPL_RELEASE(nsZipHandle)
 
-nsresult nsZipHandle::Init(nsILocalFile *file, nsZipHandle **ret)
+nsresult nsZipHandle::Init(nsIFile *file, nsZipHandle **ret, PRFileDesc **aFd)
 {
   mozilla::AutoFDClose fd;
-  nsresult rv = file->OpenNSPRFileDesc(PR_RDONLY, 0000, &fd);
+  int32_t flags = PR_RDONLY;
+#if defined(XP_WIN)
+  flags |= nsIFile::OS_READAHEAD;
+#endif
+  nsresult rv = file->OpenNSPRFileDesc(flags, 0000, &fd.rwget());
   if (NS_FAILED(rv))
     return rv;
 
-  PRInt64 size = PR_Available64(fd);
-  if (size >= PR_INT32_MAX)
+  int64_t size = PR_Available64(fd);
+  if (size >= INT32_MAX)
     return NS_ERROR_FILE_TOO_BIG;
 
   PRFileMap *map = PR_CreateFileMap(fd, size, PR_PROT_READONLY);
   if (!map)
     return NS_ERROR_FAILURE;
   
-  PRUint8 *buf = (PRUint8*) PR_MemMap(map, 0, (PRUint32) size);
+  uint8_t *buf = (uint8_t*) PR_MemMap(map, 0, (uint32_t) size);
   // Bug 525755: PR_MemMap fails when fd points at something other than a normal file.
   if (!buf) {
     PR_CloseFileMap(map);
@@ -201,16 +199,21 @@ nsresult nsZipHandle::Init(nsILocalFile *file, nsZipHandle **ret)
 
   nsRefPtr<nsZipHandle> handle = new nsZipHandle();
   if (!handle) {
-    PR_MemUnmap(buf, (PRUint32) size);
+    PR_MemUnmap(buf, (uint32_t) size);
     PR_CloseFileMap(map);
     return NS_ERROR_OUT_OF_MEMORY;
   }
 
+#if defined(XP_WIN)
+  if (aFd) {
+    *aFd = fd.forget();
+  }
+#endif
   handle->mMap = map;
-  handle->mFile = file;
-  handle->mLen = (PRUint32) size;
+  handle->mFile.Init(file);
+  handle->mLen = (uint32_t) size;
   handle->mFileData = buf;
-  *ret = handle.forget().get();
+  handle.forget(ret);
   return NS_OK;
 }
 
@@ -221,29 +224,35 @@ nsresult nsZipHandle::Init(nsZipArchive *zip, const char *entry,
   if (!handle)
     return NS_ERROR_OUT_OF_MEMORY;
 
-  handle->mBuf = new nsZipItemPtr<PRUint8>(zip, entry);
+  handle->mBuf = new nsZipItemPtr<uint8_t>(zip, entry);
   if (!handle->mBuf)
     return NS_ERROR_OUT_OF_MEMORY;
 
   if (!handle->mBuf->Buffer())
     return NS_ERROR_UNEXPECTED;
 
-  handle->mMap = nsnull;
+  handle->mMap = nullptr;
+  handle->mFile.Init(zip, entry);
   handle->mLen = handle->mBuf->Length();
   handle->mFileData = handle->mBuf->Buffer();
-  *ret = handle.forget().get();
+  handle.forget(ret);
   return NS_OK;
 }
 
+int64_t nsZipHandle::SizeOfMapping()
+{
+    return mLen;
+}
+
 nsZipHandle::~nsZipHandle()
 {
   if (mMap) {
     PR_MemUnmap((void *)mFileData, mLen);
     PR_CloseFileMap(mMap);
   }
-  mFileData = nsnull;
-  mMap = nsnull;
-  mBuf = nsnull;
+  mFileData = nullptr;
+  mMap = nullptr;
+  mBuf = nullptr;
   MOZ_COUNT_DTOR(nsZipHandle);
 }
 
@@ -254,7 +263,7 @@ nsZipHandle::~nsZipHandle()
 //---------------------------------------------
 //  nsZipArchive::OpenArchive
 //---------------------------------------------
-nsresult nsZipArchive::OpenArchive(nsZipHandle *aZipHandle)
+nsresult nsZipArchive::OpenArchive(nsZipHandle *aZipHandle, PRFileDesc *aFd)
 {
   mFd = aZipHandle;
 
@@ -262,43 +271,31 @@ nsresult nsZipArchive::OpenArchive(nsZipHandle *aZipHandle)
   PL_INIT_ARENA_POOL(&mArena, "ZipArena", ZIP_ARENABLOCKSIZE);
 
   //-- get table of contents for archive
-  nsresult rv = BuildFileList();
-  char *env = PR_GetEnv("MOZ_JAR_LOG_DIR");
-  if (env && NS_SUCCEEDED(rv) && aZipHandle->mFile) {
-    nsCOMPtr<nsILocalFile> logFile;
-    nsresult rv2 = NS_NewLocalFile(NS_ConvertUTF8toUTF16(env), PR_FALSE, getter_AddRefs(logFile));
-    
-    if (!NS_SUCCEEDED(rv2))
-      return rv;
-
-    // Create a directory for the log (in case it doesn't exist)
-    logFile->Create(nsIFile::DIRECTORY_TYPE, 0700);
-
-    nsAutoString name;
-    aZipHandle->mFile->GetLeafName(name);
-    name.Append(NS_LITERAL_STRING(".log"));
-    logFile->Append(name);
-
-    PRFileDesc* fd;
-    rv2 = logFile->OpenNSPRFileDesc(PR_WRONLY|PR_CREATE_FILE|PR_APPEND, 0644, &fd);
-    if (NS_SUCCEEDED(rv2))
-      mLog = fd;
+  nsresult rv = BuildFileList(aFd);
+  if (NS_SUCCEEDED(rv)) {
+    if (aZipHandle->mFile)
+      aZipHandle->mFile.GetURIString(mURI);
   }
   return rv;
 }
 
 nsresult nsZipArchive::OpenArchive(nsIFile *aFile)
 {
-  nsresult rv;
-  nsCOMPtr<nsILocalFile> localFile = do_QueryInterface(aFile, &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
-
   nsRefPtr<nsZipHandle> handle;
-  rv = nsZipHandle::Init(localFile, getter_AddRefs(handle));
+#if defined(XP_WIN)
+  mozilla::AutoFDClose fd;
+  nsresult rv = nsZipHandle::Init(aFile, getter_AddRefs(handle), &fd.rwget());
+#else
+  nsresult rv = nsZipHandle::Init(aFile, getter_AddRefs(handle));
+#endif
   if (NS_FAILED(rv))
     return rv;
 
+#if defined(XP_WIN)
+  return OpenArchive(handle, fd.get());
+#else
   return OpenArchive(handle);
+#endif
 }
 
 //---------------------------------------------
@@ -341,7 +338,7 @@ nsresult nsZipArchive::CloseArchive()
 {
   if (mFd) {
     PL_FinishArenaPool(&mArena);
-    mFd = NULL;
+    mFd = nullptr;
   }
 
   // CAUTION:
@@ -362,7 +359,7 @@ nsresult nsZipArchive::CloseArchive()
 nsZipItem*  nsZipArchive::GetItem(const char * aEntryName)
 {
   if (aEntryName) {
-    PRUint32 len = strlen(aEntryName);
+    uint32_t len = strlen(aEntryName);
     //-- If the request is for a directory, make sure that synthetic entries 
     //-- are created for the directories without their own entry.
     if (!mBuiltSynthetics) {
@@ -377,20 +374,15 @@ MOZ_WIN_MEM_TRY_BEGIN
       if ((len == item->nameLength) && 
           (!memcmp(aEntryName, item->Name(), len))) {
         
-        if (mLog) {
-          // Successful GetItem() is a good indicator that the file is about to be read
-          char *tmp = PL_strdup(aEntryName);
-          tmp[len]='\n';
-          PR_Write(mLog, tmp, len+1);
-          PL_strfree(tmp);
-        }
+        // Successful GetItem() is a good indicator that the file is about to be read
+        zipLog.Write(mURI, aEntryName);
         return item; //-- found it
       }
       item = item->next;
     }
-MOZ_WIN_MEM_TRY_CATCH(return nsnull)
+MOZ_WIN_MEM_TRY_CATCH(return nullptr)
   }
-  return nsnull;
+  return nullptr;
 }
 
 //---------------------------------------------
@@ -419,8 +411,8 @@ nsresult nsZipArchive::ExtractFile(nsZipItem *item, const char *outname,
   nsresult rv = NS_OK;
 
   while (true) {
-    PRUint32 count = 0;
-    PRUint8* buf = cursor.Read(&count);
+    uint32_t count = 0;
+    uint8_t* buf = cursor.Read(&count);
     if (!buf) {
       rv = NS_ERROR_FILE_CORRUPTED;
       break;
@@ -451,16 +443,16 @@ nsresult nsZipArchive::ExtractFile(nsZipItem *item, const char *outname,
 //---------------------------------------------
 // nsZipArchive::FindInit
 //---------------------------------------------
-PRInt32
+nsresult
 nsZipArchive::FindInit(const char * aPattern, nsZipFind **aFind)
 {
   if (!aFind)
     return NS_ERROR_ILLEGAL_VALUE;
 
   // null out param in case an error happens
-  *aFind = NULL;
+  *aFind = nullptr;
 
-  PRBool  regExp = PR_FALSE;
+  bool    regExp = false;
   char*   pattern = 0;
 
   // Create synthetic directory entries on demand
@@ -477,16 +469,16 @@ nsZipArchive::FindInit(const char * aPattern, nsZipFind **aFind)
         return NS_ERROR_ILLEGAL_VALUE;
 
       case NON_SXP:
-        regExp = PR_FALSE;
+        regExp = false;
         break;
 
       case VALID_SXP:
-        regExp = PR_TRUE;
+        regExp = true;
         break;
 
       default:
         // undocumented return value from RegExpValid!
-        PR_ASSERT(PR_FALSE);
+        PR_ASSERT(false);
         return NS_ERROR_ILLEGAL_VALUE;
     }
 
@@ -509,7 +501,7 @@ nsZipArchive::FindInit(const char * aPattern, nsZipFind **aFind)
 //---------------------------------------------
 // nsZipFind::FindNext
 //---------------------------------------------
-nsresult nsZipFind::FindNext(const char ** aResult, PRUint16 *aNameLen)
+nsresult nsZipFind::FindNext(const char ** aResult, uint16_t *aNameLen)
 {
   if (!mArchive || !aResult || !aNameLen)
     return NS_ERROR_ILLEGAL_VALUE;
@@ -523,17 +515,17 @@ MOZ_WIN_MEM_TRY_BEGIN
     // move to next in current chain, or move to new slot
     mItem = mItem ? mItem->next : mArchive->mFiles[mSlot];
 
-    PRBool found = PR_FALSE;
+    bool found = false;
     if (!mItem)
       ++mSlot;                          // no more in this chain, move to next slot
     else if (!mPattern)
-      found = PR_TRUE;            // always match
+      found = true;            // always match
     else if (mRegExp)
     {
       char buf[kMaxNameLength+1];
       memcpy(buf, mItem->Name(), mItem->nameLength);
       buf[mItem->nameLength]='\0';
-      found = (NS_WildCardMatch(buf, mPattern, PR_FALSE) == MATCH);
+      found = (NS_WildCardMatch(buf, mPattern, false) == MATCH);
     }
     else
       found = ((mItem->nameLength == strlen(mPattern)) &&
@@ -560,7 +552,7 @@ static nsresult ResolveSymlink(const char *path)
     return NS_ERROR_FILE_DISK_FULL;
 
   char buf[PATH_MAX+1];
-  PRInt32 length = PR_Read(fIn, (void*)buf, PATH_MAX);
+  int32_t length = PR_Read(fIn, (void*)buf, PATH_MAX);
   PR_Close(fIn);
 
   if ( (length <= 0)
@@ -591,19 +583,27 @@ nsZipItem* nsZipArchive::CreateZipItem()
 //---------------------------------------------
 //  nsZipArchive::BuildFileList
 //---------------------------------------------
-nsresult nsZipArchive::BuildFileList()
+nsresult nsZipArchive::BuildFileList(PRFileDesc *aFd)
 {
-#ifndef XP_WIN
-  NS_TIME_FUNCTION;
-#endif
   // Get archive size using end pos
-  const PRUint8* buf;
-  const PRUint8* startp = mFd->mFileData;
-  const PRUint8* endp = startp + mFd->mLen;
+  const uint8_t* buf;
+  const uint8_t* startp = mFd->mFileData;
+  const uint8_t* endp = startp + mFd->mLen;
 MOZ_WIN_MEM_TRY_BEGIN
-  PRUint32 centralOffset = 4;
+  uint32_t centralOffset = 4;
   if (mFd->mLen > ZIPCENTRAL_SIZE && xtolong(startp + centralOffset) == CENTRALSIG) {
     // Success means optimized jar layout from bug 559961 is in effect
+    uint32_t readaheadLength = xtolong(startp);
+    if (readaheadLength) {
+#if defined(XP_UNIX)
+      madvise(const_cast<uint8_t*>(startp), readaheadLength, MADV_WILLNEED);
+#elif defined(XP_WIN)
+      if (aFd) {
+        HANDLE hFile = (HANDLE) PR_FileDesc2NativeHandle(aFd);
+        mozilla::ReadAhead(hFile, 0, readaheadLength);
+      }
+#endif
+    }
   } else {
     for (buf = endp - ZIPEND_SIZE; buf > startp; buf--)
       {
@@ -619,8 +619,8 @@ MOZ_WIN_MEM_TRY_BEGIN
 
   //-- Read the central directory headers
   buf = startp + centralOffset;
-  PRUint32 sig = 0;
-  while (buf + PRInt32(sizeof(PRUint32)) <= endp &&
+  uint32_t sig = 0;
+  while (buf + int32_t(sizeof(uint32_t)) <= endp &&
          (sig = xtolong(buf)) == CENTRALSIG) {
     // Make sure there is enough data available.
     if (endp - buf < ZIPCENTRAL_SIZE)
@@ -629,17 +629,20 @@ MOZ_WIN_MEM_TRY_BEGIN
     // Read the fixed-size data.
     ZipCentral* central = (ZipCentral*)buf;
 
-    PRUint16 namelen = xtoint(central->filename_len);
-    PRUint16 extralen = xtoint(central->extrafield_len);
-    PRUint16 commentlen = xtoint(central->commentfield_len);
+    uint16_t namelen = xtoint(central->filename_len);
+    uint16_t extralen = xtoint(central->extrafield_len);
+    uint16_t commentlen = xtoint(central->commentfield_len);
 
     // Point to the next item at the top of loop
     buf += ZIPCENTRAL_SIZE + namelen + extralen + commentlen;
 
     // Sanity check variable sizes and refuse to deal with
     // anything too big: it's likely a corrupt archive.
-    if (namelen > kMaxNameLength || buf >= endp)
+    if (namelen < 1 ||
+        namelen > kMaxNameLength ||
+        buf >= endp) {
       return NS_ERROR_FILE_CORRUPTED;
+    }
 
     nsZipItem* item = CreateZipItem();
     if (!item)
@@ -650,7 +653,7 @@ MOZ_WIN_MEM_TRY_BEGIN
     item->isSynthetic = false;
 
     // Add item to file table
-    PRUint32 hash = HashName(item->Name(), namelen);
+    uint32_t hash = HashName(item->Name(), namelen);
     item->next = mFiles[hash];
     mFiles[hash] = item;
 
@@ -660,6 +663,18 @@ MOZ_WIN_MEM_TRY_BEGIN
   if (sig != ENDSIG)
     return NS_ERROR_FILE_CORRUPTED;
 
+  // Make the comment available for consumers.
+  if (endp - buf >= ZIPEND_SIZE) {
+    ZipEnd *zipend = (ZipEnd *)buf;
+
+    buf += ZIPEND_SIZE;
+    uint16_t commentlen = xtoint(zipend->commentfield_len);
+    if (endp - buf >= commentlen) {
+      mCommentPtr = (const char *)buf;
+      mCommentLen = commentlen;
+    }
+  }
+
 MOZ_WIN_MEM_TRY_CATCH(return NS_ERROR_FAILURE)
   return NS_OK;
 }
@@ -678,7 +693,7 @@ MOZ_WIN_MEM_TRY_BEGIN
   // Do this when all ziptable has scanned to prevent double entries.
   for (int i = 0; i < ZIP_TABSIZE; ++i)
   {
-    for (nsZipItem* item = mFiles[i]; item != 0; item = item->next)
+    for (nsZipItem* item = mFiles[i]; item != nullptr; item = item->next)
     {
       if (item->isSynthetic)
         continue;
@@ -689,23 +704,29 @@ MOZ_WIN_MEM_TRY_BEGIN
       //-- create already exists
       //-- start just before the last char so as to not add the item
       //-- twice if it's a directory
-      PRUint16 namelen = item->nameLength;
+      uint16_t namelen = item->nameLength;
+      MOZ_ASSERT(namelen > 0, "Attempt to build synthetic for zero-length entry name!");
       const char *name = item->Name();
-      for (PRUint16 dirlen = namelen - 1; dirlen > 0; dirlen--)
+      for (uint16_t dirlen = namelen - 1; dirlen > 0; dirlen--)
       {
         if (name[dirlen-1] != '/')
           continue;
 
+        // The character before this is '/', so if this is also '/' then we
+        // have an empty path component. Skip it.
+        if (name[dirlen] == '/')
+          continue;
+
         // Is the directory already in the file table?
-        PRUint32 hash = HashName(item->Name(), dirlen);
-        PRBool found = PR_FALSE;
-        for (nsZipItem* zi = mFiles[hash]; zi != NULL; zi = zi->next)
+        uint32_t hash = HashName(item->Name(), dirlen);
+        bool found = false;
+        for (nsZipItem* zi = mFiles[hash]; zi != nullptr; zi = zi->next)
         {
           if ((dirlen == zi->nameLength) &&
               (0 == memcmp(item->Name(), zi->Name(), dirlen)))
           {
             // we've already added this dir and all its parents
-            found = PR_TRUE;
+            found = true;
             break;
           }
         }
@@ -737,29 +758,29 @@ MOZ_WIN_MEM_TRY_CATCH(return NS_ERROR_FAILURE)
 nsZipHandle* nsZipArchive::GetFD()
 {
   if (!mFd)
-    return NULL;
+    return nullptr;
   return mFd.get();
 }
 
 //---------------------------------------------
 // nsZipArchive::GetData
 //---------------------------------------------
-const PRUint8* nsZipArchive::GetData(nsZipItem* aItem)
+const uint8_t* nsZipArchive::GetData(nsZipItem* aItem)
 {
   PR_ASSERT (aItem);
 MOZ_WIN_MEM_TRY_BEGIN
   //-- read local header to get variable length values and calculate
   //-- the real data offset
-  PRUint32 len = mFd->mLen;
-  const PRUint8* data = mFd->mFileData;
-  PRUint32 offset = aItem->LocalOffset();
+  uint32_t len = mFd->mLen;
+  const uint8_t* data = mFd->mFileData;
+  uint32_t offset = aItem->LocalOffset();
   if (offset + ZIPLOCAL_SIZE > len)
-    return nsnull;
+    return nullptr;
 
   // -- check signature before using the structure, in case the zip file is corrupt
   ZipLocal* Local = (ZipLocal*)(data + offset);
   if ((xtolong(Local->signature) != LOCALSIG))
-    return nsnull;
+    return nullptr;
 
   //-- NOTE: extralen is different in central header and local header
   //--       for archives created using the Unix "zip" utility. To set
@@ -770,30 +791,55 @@ MOZ_WIN_MEM_TRY_BEGIN
 
   // -- check if there is enough source data in the file
   if (offset + aItem->Size() > len)
-    return nsnull;
+    return nullptr;
 
   return data + offset;
-MOZ_WIN_MEM_TRY_CATCH(return nsnull)
+MOZ_WIN_MEM_TRY_CATCH(return nullptr)
+}
+
+// nsZipArchive::GetComment
+bool nsZipArchive::GetComment(nsACString &aComment)
+{
+MOZ_WIN_MEM_TRY_BEGIN
+  aComment.Assign(mCommentPtr, mCommentLen);
+MOZ_WIN_MEM_TRY_CATCH(return false)
+  return true;
+}
+
+//---------------------------------------------
+// nsZipArchive::SizeOfMapping
+//---------------------------------------------
+int64_t nsZipArchive::SizeOfMapping()
+{
+    return mFd ? mFd->SizeOfMapping() : 0;
 }
 
 //------------------------------------------
 // nsZipArchive constructor and destructor
 //------------------------------------------
 
-nsZipArchive::nsZipArchive() :
-  mBuiltSynthetics(false)
+nsZipArchive::nsZipArchive()
+  : mRefCnt(0)
+  , mBuiltSynthetics(false)
 {
+  zipLog.AddRef();
+
   MOZ_COUNT_CTOR(nsZipArchive);
 
-  // initialize the table to NULL
+  // initialize the table to nullptr
   memset(mFiles, 0, sizeof(mFiles));
 }
 
+NS_IMPL_ADDREF(nsZipArchive)
+NS_IMPL_RELEASE(nsZipArchive)
+
 nsZipArchive::~nsZipArchive()
 {
   CloseArchive();
 
   MOZ_COUNT_DTOR(nsZipArchive);
+
+  zipLog.Release();
 }
 
 
@@ -801,7 +847,7 @@ nsZipArchive::~nsZipArchive()
 // nsZipFind constructor and destructor
 //------------------------------------------
 
-nsZipFind::nsZipFind(nsZipArchive* aZip, char* aPattern, PRBool aRegExp) : 
+nsZipFind::nsZipFind(nsZipArchive* aZip, char* aPattern, bool aRegExp) : 
   mArchive(aZip),
   mPattern(aPattern),
   mItem(0),
@@ -827,13 +873,13 @@ nsZipFind::~nsZipFind()
  *
  * returns a hash key for the entry name 
  */
-static PRUint32 HashName(const char* aName, PRUint16 len)
+static uint32_t HashName(const char* aName, uint16_t len)
 {
   PR_ASSERT(aName != 0);
 
-  const PRUint8* p = (const PRUint8*)aName;
-  const PRUint8* endp = p + len;
-  PRUint32 val = 0;
+  const uint8_t* p = (const uint8_t*)aName;
+  const uint8_t* endp = p + len;
+  uint32_t val = 0;
   while (p != endp) {
     val = val*37 + *p++;
   }
@@ -847,9 +893,9 @@ static PRUint32 HashName(const char* aName, PRUint16 len)
  *  Converts a two byte ugly endianed integer
  *  to our platform's integer.
  */
-static PRUint16 xtoint (const PRUint8 *ii)
+static uint16_t xtoint (const uint8_t *ii)
 {
-  return (PRUint16) ((ii [0]) | (ii [1] << 8));
+  return (uint16_t) ((ii [0]) | (ii [1] << 8));
 }
 
 /*
@@ -858,9 +904,9 @@ static PRUint16 xtoint (const PRUint8 *ii)
  *  Converts a four byte ugly endianed integer
  *  to our platform's integer.
  */
-static PRUint32 xtolong (const PRUint8 *ll)
+static uint32_t xtolong (const uint8_t *ll)
 {
-  return (PRUint32)( (ll [0] <<  0) |
+  return (uint32_t)( (ll [0] <<  0) |
                      (ll [1] <<  8) |
                      (ll [2] << 16) |
                      (ll [3] << 24) );
@@ -871,7 +917,7 @@ static PRUint32 xtolong (const PRUint8 *ll)
  *
  * returns last modification time in microseconds
  */
-static PRTime GetModTime(PRUint16 aDate, PRUint16 aTime)
+static PRTime GetModTime(uint16_t aDate, uint16_t aTime)
 {
   // Note that on DST shift we can't handle correctly the hour that is valid
   // in both DST zones
@@ -898,37 +944,37 @@ static PRTime GetModTime(PRUint16 aDate, PRUint16 aTime)
   return PR_ImplodeTime(&time);
 }
 
-PRUint32 nsZipItem::LocalOffset()
+uint32_t nsZipItem::LocalOffset()
 {
   return xtolong(central->localhdr_offset);
 }
 
-PRUint32 nsZipItem::Size()
+uint32_t nsZipItem::Size()
 {
   return isSynthetic ? 0 : xtolong(central->size);
 }
 
-PRUint32 nsZipItem::RealSize()
+uint32_t nsZipItem::RealSize()
 {
   return isSynthetic ? 0 : xtolong(central->orglen);
 }
 
-PRUint32 nsZipItem::CRC32()
+uint32_t nsZipItem::CRC32()
 {
   return isSynthetic ? 0 : xtolong(central->crc32);
 }
 
-PRUint16 nsZipItem::Date()
+uint16_t nsZipItem::Date()
 {
   return isSynthetic ? kSyntheticDate : xtoint(central->date);
 }
 
-PRUint16 nsZipItem::Time()
+uint16_t nsZipItem::Time()
 {
   return isSynthetic ? kSyntheticTime : xtoint(central->time);
 }
 
-PRUint16 nsZipItem::Compression()
+uint16_t nsZipItem::Compression()
 {
   return isSynthetic ? STORED : xtoint(central->method);
 }
@@ -938,21 +984,21 @@ bool nsZipItem::IsDirectory()
   return isSynthetic || ((nameLength > 0) && ('/' == Name()[nameLength - 1]));
 }
 
-PRUint16 nsZipItem::Mode()
+uint16_t nsZipItem::Mode()
 {
   if (isSynthetic) return 0755;
-  return ((PRUint16)(central->external_attributes[2]) | 0x100);
+  return ((uint16_t)(central->external_attributes[2]) | 0x100);
 }
 
-const PRUint8 * nsZipItem::GetExtraField(PRUint16 aTag, PRUint16 *aBlockSize)
+const uint8_t * nsZipItem::GetExtraField(uint16_t aTag, uint16_t *aBlockSize)
 {
-  if (isSynthetic) return nsnull;
+  if (isSynthetic) return nullptr;
 MOZ_WIN_MEM_TRY_BEGIN
   const unsigned char *buf = ((const unsigned char*)central) + ZIPCENTRAL_SIZE +
                              nameLength;
-  PRUint32 buflen = (PRUint32)xtoint(central->extrafield_len);
-  PRUint32 pos = 0;
-  PRUint16 tag, blocksize;
+  uint32_t buflen = (uint32_t)xtoint(central->extrafield_len);
+  uint32_t pos = 0;
+  uint16_t tag, blocksize;
 
   while (buf && (pos + 4) <= buflen) {
     tag = xtoint(buf + pos);
@@ -966,8 +1012,8 @@ MOZ_WIN_MEM_TRY_BEGIN
     pos += blocksize + 4;
   }
 
-MOZ_WIN_MEM_TRY_CATCH(return nsnull)
-  return nsnull;
+MOZ_WIN_MEM_TRY_CATCH(return nullptr)
+  return nullptr;
 }
 
 
@@ -976,8 +1022,8 @@ PRTime nsZipItem::LastModTime()
   if (isSynthetic) return GetModTime(kSyntheticDate, kSyntheticTime);
 
   // Try to read timestamp from extra field
-  PRUint16 blocksize;
-  const PRUint8 *tsField = GetExtraField(EXTENDED_TIMESTAMP_FIELD, &blocksize);
+  uint16_t blocksize;
+  const uint8_t *tsField = GetExtraField(EXTENDED_TIMESTAMP_FIELD, &blocksize);
   if (tsField && blocksize >= 5 && tsField[4] & EXTENDED_TIMESTAMP_MODTIME) {
     return (PRTime)(xtolong(tsField + 5)) * PR_USEC_PER_SEC;
   }
@@ -993,7 +1039,7 @@ bool nsZipItem::IsSymlink()
 }
 #endif
 
-nsZipCursor::nsZipCursor(nsZipItem *item, nsZipArchive *aZip, PRUint8* aBuf, PRUint32 aBufSize, bool doCRC) :
+nsZipCursor::nsZipCursor(nsZipItem *item, nsZipArchive *aZip, uint8_t* aBuf, uint32_t aBufSize, bool doCRC) :
   mItem(item),
   mBuf(aBuf),
   mBufSize(aBufSize),
@@ -1022,20 +1068,27 @@ nsZipCursor::~nsZipCursor()
   }
 }
 
-PRUint8* nsZipCursor::Read(PRUint32 *aBytesRead) {
+uint8_t* nsZipCursor::ReadOrCopy(uint32_t *aBytesRead, bool aCopy) {
   int zerr;
-  PRUint8 *buf = nsnull;
+  uint8_t *buf = nullptr;
   bool verifyCRC = true;
 
   if (!mZs.next_in)
-    return nsnull;
+    return nullptr;
 MOZ_WIN_MEM_TRY_BEGIN
   switch (mItem->Compression()) {
   case STORED:
-    *aBytesRead = mZs.avail_in;
-    buf = mZs.next_in;
-    mZs.next_in += mZs.avail_in;
-    mZs.avail_in = 0;
+    if (!aCopy) {
+      *aBytesRead = mZs.avail_in;
+      buf = mZs.next_in;
+      mZs.next_in += mZs.avail_in;
+      mZs.avail_in = 0;
+    } else {
+      *aBytesRead = mZs.avail_in > mBufSize ? mBufSize : mZs.avail_in;
+      memcpy(mBuf, mZs.next_in, *aBytesRead);
+      mZs.avail_in -= *aBytesRead;
+      mZs.next_in += *aBytesRead;
+    }
     break;
   case DEFLATED:
     buf = mBuf;
@@ -1044,26 +1097,26 @@ MOZ_WIN_MEM_TRY_BEGIN
     
     zerr = inflate(&mZs, Z_PARTIAL_FLUSH);
     if (zerr != Z_OK && zerr != Z_STREAM_END)
-      return nsnull;
+      return nullptr;
     
     *aBytesRead = mZs.next_out - buf;
     verifyCRC = (zerr == Z_STREAM_END);
     break;
   default:
-    return nsnull;
+    return nullptr;
   }
 
   if (mDoCRC) {
     mCRC = crc32(mCRC, (const unsigned char*)buf, *aBytesRead);
     if (verifyCRC && mCRC != mItem->CRC32())
-      return nsnull;
+      return nullptr;
   }
-MOZ_WIN_MEM_TRY_CATCH(return nsnull)
+MOZ_WIN_MEM_TRY_CATCH(return nullptr)
   return buf;
 }
 
 nsZipItemPtr_base::nsZipItemPtr_base(nsZipArchive *aZip, const char * aEntryName, bool doCRC) :
-  mReturnBuf(nsnull)
+  mReturnBuf(nullptr)
 {
   // make sure the ziparchive hangs around
   mZipHandle = aZip->GetFD();
@@ -1072,24 +1125,24 @@ nsZipItemPtr_base::nsZipItemPtr_base(nsZipArchive *aZip, const char * aEntryName
   if (!item)
     return;
 
-  PRUint32 size = 0;
+  uint32_t size = 0;
   if (item->Compression() == DEFLATED) {
     size = item->RealSize();
-    mAutoBuf = new PRUint8[size];
+    mAutoBuf = new ((fallible_t())) uint8_t[size];
+    if (!mAutoBuf) {
+      return;
+    }
   }
 
   nsZipCursor cursor(item, aZip, mAutoBuf, size, doCRC);
   mReturnBuf = cursor.Read(&mReadlen);
   if (!mReturnBuf) {
-    Telemetry::Accumulate(Telemetry::ZIPARCHIVE_CRC, false);
     return;
   }
 
   if (mReadlen != item->RealSize()) {
     NS_ASSERTION(mReadlen == item->RealSize(), "nsZipCursor underflow");
-    mReturnBuf = nsnull;
+    mReturnBuf = nullptr;
     return;
   }
-
-  Telemetry::Accumulate(Telemetry::ZIPARCHIVE_CRC, true);
 }
