@@ -1,4 +1,4 @@
-do_load_httpd_js();
+Cu.import("resource://testing-common/httpd.js");
 
 var server;
 const BUGID = "263127";
@@ -34,13 +34,14 @@ var listener = {
 
 function run_test() {
   // start server
-  server = new nsHttpServer();
-  server.start(4444);
+  server = new HttpServer();
+  server.start(-1);
 
   // Initialize downloader
   var channel = Cc["@mozilla.org/network/io-service;1"]
                   .getService(Ci.nsIIOService)
-                  .newChannel("http://localhost:4444/", null, null);
+                  .newChannel("http://localhost:" +
+                              server.identity.primaryPort + "/", null, null);
 
   var targetFile = Cc["@mozilla.org/file/directory_service;1"]
                      .getService(Ci.nsIProperties)
