@@ -1,6 +1,3 @@
-const Cc = Components.classes;
-const Ci = Components.interfaces;
-
 function run_test() {
   const ios = Cc["@mozilla.org/network/io-service;1"].
     getService(Ci.nsIIOService);
