@@ -508,7 +508,7 @@ MarionetteServerConnection.prototype = {
       let win = this.getCurrentWindow();
       if (!win ||
           (appName == "IceCat" && !win.gBrowser) ||
-          (appName == "Fennec" && !win.BrowserApp)) {
+          (appName == "IceCatMobile" && !win.BrowserApp)) {
         checkTimer.initWithCallback(waitForWindow.bind(this), 100,
                                     Ci.nsITimer.TYPE_ONE_SHOT);
       }
@@ -2542,7 +2542,7 @@ BrowserObj.prototype = {
       case "IceCat":
         this.browser = win.gBrowser;
         break;
-      case "Fennec":
+      case "IceCatMobile":
         this.browser = win.BrowserApp;
         break;
     }
