@@ -31,7 +31,7 @@ class DeviceManagerADB(DeviceManager):
     _tempDir = None
     default_timeout = 300
 
-    def __init__(self, host=None, port=5555, retryLimit=5, packageName='fennec',
+    def __init__(self, host=None, port=5555, retryLimit=5, packageName='icecatmobile',
                  adbPath='adb', deviceSerial=None, deviceRoot=None,
                  logLevel=mozlog.ERROR, **kwargs):
         DeviceManager.__init__(self, logLevel)
@@ -47,11 +47,11 @@ class DeviceManagerADB(DeviceManager):
         # where multiple devices are being managed by the same adb instance.
         self._deviceSerial = deviceSerial
 
-        if packageName == 'fennec':
+        if packageName == 'icecatmobile':
             if os.getenv('USER'):
-                self._packageName = 'org.mozilla.fennec_' + os.getenv('USER')
+                self._packageName = 'org.mozilla.icecatmobile_' + os.getenv('USER')
             else:
-                self._packageName = 'org.mozilla.fennec_'
+                self._packageName = 'org.mozilla.icecatmobile_'
         elif packageName:
             self._packageName = packageName
 
