@@ -240,7 +240,7 @@ class AndroidXPCShellRunner(MozbuildObject):
         options.setup = not no_setup
         options.keepGoing = keep_going
         options.objdir = self.topobjdir
-        options.localLib = os.path.join(self.topobjdir, 'dist/fennec')
+        options.localLib = os.path.join(self.topobjdir, 'dist/icecatmobile')
         options.localBin = os.path.join(self.topobjdir, 'dist/bin')
         options.testingModulesDir = os.path.join(self.topobjdir, '_tests/modules')
         options.mozInfo = os.path.join(self.topobjdir, 'mozinfo.json')
@@ -250,7 +250,7 @@ class AndroidXPCShellRunner(MozbuildObject):
             options.localAPK = local_apk
         else:
             for file in os.listdir(os.path.join(options.objdir, "dist")):
-                if file.endswith(".apk") and file.startswith("fennec"):
+                if file.endswith(".apk") and file.startswith("icecatmobile"):
                     options.localAPK = os.path.join(options.objdir, "dist")
                     options.localAPK = os.path.join(options.localAPK, file)
                     print ("using APK: " + options.localAPK)
@@ -417,7 +417,7 @@ class MachCommands(MachCommandBase):
     @CommandArgument('--no-setup', action='store_true',
         help='(Android) Do not copy files to device')
     @CommandArgument('--local-apk', type=str, default=None,
-        help='(Android) Use specified Fennec APK')
+        help='(Android) Use specified IceCatMobile APK')
     @CommandArgument('--busybox', type=str, default=None,
         help='(B2G) Path to busybox binary (speeds up installation of tests).')
     def run_xpcshell_test(self, **params):
